/*

 * $Id: AbstractSite.java,v 1.1.1.1 2000/05/15 22:11:02 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 *

 */



package diva.canvas;



import diva.canvas.Site;

import diva.canvas.TransformContext;

import diva.canvas.Figure;

import java.awt.geom.Point2D;



/** An abstract implementation of Site. This class provides default

 * implementations of several methods in the Site interface, to

 * make it easier to implement site classes.

 *

 * @version	$Revision: 1.1.1.1 $

 * @author 	John Reekie

 */

public abstract class AbstractSite implements Site {



    /** The normal of the site. This is 0.0 by default.

     */

    private double _normal = 0.0;

    

    /** Get the figure to which this site is attached.

     */

    public abstract Figure getFigure ();



    /** Get the ID of this site.

     */

    public abstract int getID ();



    /** Get the angle of the normal to this site, in radians

     * between zero and 2pi. This default method returns 0.0.

     */

    public double getNormal () {

        return 0.0;

    }



    /** Get the point location of the site, in the enclosing

     * transform context.

     */

    public Point2D getPoint () {

         return new Point2D.Double(getX(), getY());

    } 



    /** Get the point location of the site, in the given

     * transform context. The given context must be an enclosing

     * context of the site.

     */

    public Point2D getPoint (TransformContext tc) {

        TransformContext current = getTransformContext();

        Point2D p = getPoint();

        while (tc != current) {

            current.getTransform().transform(p, p);

            current = current.getParent();

        }

        return p;

    }



    /** Get the enclosing transform context of this site.

     *  As a default behavior, return the transform context

     *  of the associated figure.

     */

    public TransformContext getTransformContext() {

        return getFigure().getTransformContext();

    } 

    

    /** Get the x-coordinate of the site, in the enclosing

     * transform context.

     */

    public abstract double getX ();



    /** Get the y-coordinate of the site, in the enclosing

     * transform context.

     */

    public abstract double getY ();



    /** Test if this site has a "normal" to it. This default

     * implementation returns false.

     */

    public boolean hasNormal () {

        return false;

    }



    /** Test if this site has a normal in the given direction.

     * This default implementation returns false.

     */

    public boolean isNormal (int direction) {

        return false;

    }



    /** Set the normal "out" of the site. This default implementation

     * does nothing.

     */

    public void setNormal (double normal) {

        // do nothing

    }



    /** Translate the site by the indicated distance. This

     * default implementation does nothing.

     */

    public void translate (double x, double y) {

        // do nothing

    }

}

