/*

 * $Id: CanvasUtilities.java,v 1.1.1.1 2000/05/15 22:11:02 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 *

 */



package diva.canvas;



import diva.util.java2d.Polyline2D;

import diva.util.java2d.Polygon2D;



import java.awt.Shape;

import java.awt.geom.AffineTransform;

import java.awt.geom.GeneralPath;

import java.awt.geom.Point2D;

import java.awt.geom.RectangularShape;

import java.awt.geom.Rectangle2D;

import java.awt.geom.Rectangle2D.Double;

import diva.util.FilteredIterator;

import diva.util.Filter;



import java.util.Iterator;

import javax.swing.SwingConstants;



/** A collection of canvas utilities. These utilities perform

 * useful functions related to the structural aspects of diva.canvas

 * that do not properly belong in any one class. Some of them

 * perform utility geometric functions that are not available

 * in the Java 2D API, while others accept iterators over Figures

 * or Shapes and compute a useful composite result.

 *

 * @version	$Revision: 1.1.1.1 $

 * @author 	John Reekie

 */

public final class CanvasUtilities {



    ///////////////////////////////////////////////////////////////////

    //// Constants used by transform()



    private final static int m00 = 0;

    private final static int m10 = 1;

    private final static int m01 = 2;

    private final static int m11 = 3;

    private final static int m02 = 4;

    private final static int m12 = 5;



    /** Cannot instantiate

     */

    private CanvasUtilities() {}



    /** Clone a shape. This method is needed because Shape by itself

     * does not define clone(), although many (all?) shape instances

     * do.

     */

    public static Shape clone (Shape s) {

        // FIXME Add more specific shapes

        if (s instanceof RectangularShape) {

            return (RectangularShape) ((RectangularShape) s).clone();



        } else {

            return new GeneralPath(s);

        }

    }



    /** Compute a composite shape. The iterator must contain figures,

     * from which the shapes are obtained and joined into a

     * more complex shape. If the iterator is empty, return

     * a very small rectangle.

     */

    public static Shape computeCompositeShape (Iterator i) {

        if (!i.hasNext()) {

            return new Rectangle2D.Double();

        }

        // Start the union with the shape of the first child

        Figure f = (Figure) i.next();

        GeneralPath shape = new GeneralPath(f.getShape());



        // Scan the rest of the children and take the union

        while (i.hasNext()) {

            f = (Figure) i.next();

            shape.append(f.getShape(), false); // Don't connect

        }

        return shape;

    }



    /** Compute the bounding box of a set of connectors. The iterator

     * must contain connectors.

     */

    public static Rectangle2D computeSiteBounds (Iterator i) {

        double x1, y1, x2, y2;



        Site f = (Site) i.next();

        x1 = x2 = f.getX();

        y1 = y2 = f.getY();

        while (i.hasNext()) {

            f = (Site) i.next();

            double x = f.getX();

            double y = f.getY();

            if (x < x1) {

                x1 = x;

            }

            if (x > x2) {

                x2 = x;

            }

            if (y < y1) {

                y1 = y;

            }

            if (y > y2) {

                y2 = y;

            }

        }

        return new Rectangle2D.Double(x1,y1,x2-x1,y2-y1);

    }



    /** Compute a composite bounding box. The iterator must contain

     * figures, from which the bounding boxes are obtained and joined

     * into a more complex shape. If the iterator is empty, return

     * a very small rectangle.

     */

    public static Rectangle2D computeCompositeBounds (Iterator i) {

        if (!i.hasNext()) {

            return new Rectangle2D.Double();

        }

        // Get a copy of the bounds of the first child 

        Figure f = (Figure) i.next();

        Rectangle2D b = f.getBounds();

        Rectangle2D bounds = new Rectangle2D.Double(

                b.getX(), b.getY(),

                b.getWidth(), b.getHeight());



        // Scan the rest of the children and take the union

        while (i.hasNext()) {

            f = (Figure) i.next();

            Rectangle2D.union(bounds, f.getBounds(), bounds);

        }

        return bounds;

    }



    /** Get the transform that will make the first

     * rectangle change into the second.

     */

    public static AffineTransform computeTransform(

            RectangularShape r, RectangularShape s) {

        AffineTransform at = new AffineTransform();

        at.translate(s.getX(), s.getY());

        at.scale(s.getWidth()/r.getWidth(), s.getHeight()/r.getHeight());

        at.translate(-r.getX(), -r.getY());

        return at;

    }



    /** Return the point at the center of a Rectangle.

     */

    public static Point2D getCenterPoint (Rectangle2D r) {

        return new Point2D.Double(r.getCenterX(), r.getCenterY());

    }



    /** Return the point at the center of a figure. This is

     * simple but so common it's worth having a method for it.

     */

    public static Point2D getCenterPoint (Figure f) {

        return getCenterPoint(f.getBounds());

    }



    /** Get the point on the given rectangular shape indicated by the location

     * flag. This flag must be one of those defined in javax.swing.Constants.

     */

    public static Point2D getLocation (Rectangle2D r, int location) {

        double x;

        double y;



        switch (location) {

        case SwingConstants.CENTER:

            x = r.getCenterX();

            y = r.getCenterY();

            break;



        case SwingConstants.NORTH:

            x = r.getCenterX();

            y = r.getMinY();

            break;



        case SwingConstants.NORTH_EAST:

            x = r.getMaxX();

            y = r.getMinY();

            break;

            

        case SwingConstants.EAST:

            x = r.getMaxX();

            y = r.getCenterY();

            break;

            

        case SwingConstants.SOUTH_EAST:

            x = r.getMaxX();

            y = r.getMaxY();

            break;

            

        case SwingConstants.SOUTH:

            x = r.getCenterX();

            y = r.getMaxY();

            break;

            

        case SwingConstants.SOUTH_WEST:

            x = r.getMinX();

            y = r.getMaxY();

            break;

            

        case SwingConstants.WEST:

            x = r.getMinX();

            y = r.getCenterY();

            break;

            

        case SwingConstants.NORTH_WEST:

            x = r.getMinX();

            y = r.getMinY();

            break;

            

        default:

            throw new IllegalArgumentException(

                    "Unknown location constant: " + location);

        }

        return new Point2D.Double(x, y);

    }



    /** Return true if the given transform maps a rectangle

     * to a rectangle. If this method returns true, then passing

     * a rectangle to transform() is guaranteed to return a rectangle.

     */

    public static boolean isOrthogonal (AffineTransform at) {

        int t = at.getType();

	

	// FIXME: should be:

	//return (t &

        //            ( AffineTransform.TYPE_GENERAL_ROTATION 

        //            | AffineTransform.TYPE_GENERAL_TRANSFORM)) == 0;

	

	return (t &

                ( AffineTransform.TYPE_MASK_ROTATION 

                        | AffineTransform.TYPE_GENERAL_TRANSFORM)) == 0;

    }



    /** Return the first figure that is hit by the given region.

     * The figures tested are contained in the given iterator, which

     * must contain only figures. And figure that is also a FigureContainer

     * is descended into recursively; other figures are simply tested

     * using their hit() method.

     *

     * @param i An iterator over figures

     * @param region A rectangle which represents the pick or hit region

     * @returns The topmost descendent under the pick region, or null

     * there isn't one.

     */

    public static Figure pick (Iterator i, Rectangle2D region) {

        Figure f;

        Figure picked;



        while (i.hasNext()) {

            f = (Figure) i.next();

            if ((f instanceof FigureContainer)) {

                picked = ((FigureContainer) f).pick(region);

                if (picked != null) {

                    return picked;

                }

            }

            if (f.hit(region)) {

                return f;

            }

        }

        return null;

    }





    /**

     * Return an iterator over the figures hit by the given region.

     * The figures tested are contained in the given iterator, which

     * must contain only figures. And figure that is also a FigureContainer

     * is descended into recursively; other figures are simply tested

     * using their hit() method.

     *

     * XXX how to do this?

     *

     * @param i An iterator over figures

     * @param region A rectangle which represents the pick or hit region

     * @returns An iterator over the hit figures.

     */

    public static Iterator pickIter (Iterator i, Rectangle2D region) {

        final Rectangle2D rl = region;

        

        return new FilteredIterator(i, new Filter() {

            Rectangle2D _region = rl;



            public boolean accept(Object o) {

                Figure f = (Figure)o;

                return f.hit(_region);

            }

        });

    }



    /** Reverse a direction flag. The flag must one of the eight compass directions

     * defined in SwingConstants. Return the flag that represents the opposite

     * direction.

     */

    public static int reverseDirection (int direction) {

        switch (direction) {

        case SwingConstants.CENTER:

            return SwingConstants.CENTER;



        case SwingConstants.NORTH:

            return SwingConstants.SOUTH;

            

        case SwingConstants.NORTH_EAST:

            return SwingConstants.SOUTH_WEST;

            

        case SwingConstants.EAST:

            return SwingConstants.WEST;

            

        case SwingConstants.SOUTH_EAST:

            return SwingConstants.NORTH_WEST;

            

        case SwingConstants.SOUTH:

            return SwingConstants.NORTH;

            

        case SwingConstants.SOUTH_WEST:

            return SwingConstants.NORTH_EAST;

            

        case SwingConstants.WEST:

            return SwingConstants.EAST;

            

        case SwingConstants.NORTH_WEST:

            return SwingConstants.SOUTH_EAST;

            

        default:

            throw new IllegalArgumentException(

                    "Unknown direction constant: " + direction);

        }

    }



    // FIXME FIXME

    private static boolean _transformRectangularShapeIsBroken = true;



    /** Transform a shape with the supplied transform. If the shape is

     * an instance of RectangularShape, then the transformation will

     * modify that shape and return it if it is possible to do so.

     * Otherwise, AffineTransform.createTransformedShape() is used

     * to create a new shape, which is then returned.

     */

    public static Shape transform (RectangularShape r, AffineTransform at) {

        // FIXME: should be able to deal with quadrant rotations

        if (_transformRectangularShapeIsBroken) {

            // FIXME FIXME

	    Rectangle2D bounds = at.createTransformedShape(r).getBounds2D();

	    r.setFrame(bounds);

            return r;



        } else if ((at.getType() &

                        (AffineTransform.TYPE_MASK_SCALE |

                                AffineTransform.TYPE_TRANSLATION |

                                AffineTransform.TYPE_IDENTITY)) != 0) {

            double x = r.getX();

            double y = r.getY();

            double w = r.getWidth();

            double h = r.getHeight();

            double xdash = 0.0, ydash = 0.0, wdash = 0.0, hdash = 0.0;



            double m[] = new double[6];;

            at.getMatrix(m);

  

            switch(at.getType()) {

                case (AffineTransform.TYPE_GENERAL_SCALE |

                        AffineTransform.TYPE_TRANSLATION):

                    case (AffineTransform.TYPE_UNIFORM_SCALE |

                            AffineTransform.TYPE_TRANSLATION):

                    xdash = x * m[m00] + m[m02];

                ydash = y * m[m11] + m[m12];

                wdash = w * m[m00];

                hdash = h * m[m11];

                break;



            case (AffineTransform.TYPE_GENERAL_SCALE):

            case (AffineTransform.TYPE_UNIFORM_SCALE):

                xdash = x * m[m00];

                ydash = y * m[m11];

                wdash = w * m[m00];

                hdash = h * m[m11];

                break;



            case (AffineTransform.TYPE_TRANSLATION):

                xdash = x + m[m02];

                ydash = y + m[m12];

                wdash = w;

                hdash = h;

                break;



            case (AffineTransform.TYPE_IDENTITY):

                xdash = x;

                ydash = y;

                wdash = w;

                hdash = h;

                break;

            }

            r.setFrame(xdash,ydash,wdash,hdash);

            return r;

        } else {

            return at.createTransformedShape(r);

        }

    }



    /** Transform a shape with the supplied transform. As much as

     * possible, this method attempts to preserve the type of the

     * shape and to modify it directly if possible: if the shape of

     * this figure is an instance of RectangularShape or Polyline,

     * then the shape may be modified directly. Otherwise, a general

     * transformation is used that creates and returns a new instance

     * of GeneralPath.

     */

    public static Shape transform (Shape s, AffineTransform at) {

        if (s instanceof RectangularShape) {

            return CanvasUtilities.transform((RectangularShape)s, at);



        } else if (s instanceof Polygon2D) {

            ((Polygon2D) s).transform(at);

            return s;



        } else if (s instanceof Polyline2D) {

            ((Polyline2D) s).transform(at);

            return s;



        } else if (s instanceof GeneralPath) {

            ((GeneralPath) s).transform(at);

            return s;



        } else {

            return at.createTransformedShape(s);

        }

    }



    /** Translate a figure the given distance in the direction given

     * by the flag. The flag must one of the eight compass directions

     * defined in SwingConstants.

     */

    public static void translate (Figure f, 

            double distance, int direction) {

        Point2D.Double p = new Point2D.Double();

        translate(p, distance, direction);

        f.translate(p.x, p.y);

    }



    /** Translate a point the given distance in the direction given

     * by the flag. The flag must one of the eight compass directions

     * defined in SwingConstants. Return the same point, but modified.

     */

    public static Point2D translate (Point2D p,

            double distance, int direction) {

        if (p instanceof Point2D.Double) {

            return translate((Point2D.Double) p, distance, direction);

        } else {

            return translate((Point2D.Float) p, distance, direction);

        }

    }



    /** Translate a point the given distance in the direction given

     * by the flag. The flag must one of the eight compass directions

     * defined in SwingConstants. Return the same point, but modified.

     */

    public static Point2D translate (Point2D.Double p,

            double distance, int direction) {

        switch (direction) {

        case SwingConstants.NORTH:

            p.y -= distance;

            break;

        case SwingConstants.NORTH_EAST:

            p.y -= distance;

            p.x += distance;

            break;

        case SwingConstants.EAST:

            p.x += distance;

            break;

        case SwingConstants.SOUTH_EAST:

            p.y += distance;

            p.x += distance;

            break;

        case SwingConstants.SOUTH:

            p.y += distance;

            break;

        case SwingConstants.SOUTH_WEST:

            p.y += distance;

            p.x -= distance;

            break;

        case SwingConstants.WEST:

            p.x -= distance;

            break;

        case SwingConstants.NORTH_WEST:

            p.y -= distance;

            p.x -= distance;

            break;

        default:

            throw new IllegalArgumentException(

                    "Unknown direction constant: " + direction);

        }

        return p;

    }



    /** Translate a point the given distance in the direction given

     * by the flag. The flag must one of the eight compass directions

     * defined in SwingConstants. Return the same point, but modified.

     */

    public static Point2D translate (Point2D.Float p,

            double distance, int direction) {

        switch (direction) {

        case SwingConstants.NORTH:

            p.y -= (float)distance;

            break;

        case SwingConstants.NORTH_EAST:

            p.y -= (float)distance;

            p.x += (float)distance;

            break;

        case SwingConstants.EAST:

            p.x += (float)distance;

            break;

        case SwingConstants.SOUTH_EAST:

            p.y += (float)distance;

            p.x += (float)distance;

            break;

        case SwingConstants.SOUTH:

            p.y += (float)distance;

            break;

        case SwingConstants.SOUTH_WEST:

            p.y += (float)distance;

            p.x -= (float)distance;

            break;

        case SwingConstants.WEST:

            p.x -= (float)distance;

            break;

        case SwingConstants.NORTH_WEST:

            p.y -= (float)distance;

            p.x -= (float)distance;

            break;

        default:

            throw new IllegalArgumentException(

                    "Unknown direction constant: " + direction);

        }

        return p;

    }



    /** Translate a shape the given distance. If it is possible to do

     * so, perform this translation efficiently, modifying and returning

     * the passed shape. Otherwise use

     * AffineTransform.createTransformedShape() to create a new translated

     * shape, and return that.

     */

    public static Shape translate (

            Shape s, double x, double y) {

        if (s instanceof RectangularShape) {

            RectangularShape r = (RectangularShape) s;

            r.setFrame(x+r.getX(), y+r.getY(), r.getWidth(), r.getHeight());

            return r;



        } else if (s instanceof Polygon2D) {

            ((Polygon2D) s).translate(x,y);

            return s;



        } else if (s instanceof Polyline2D) {

            ((Polyline2D) s).translate(x,y);

            return s;



        } else if (s instanceof GeneralPath) {

            AffineTransform at = AffineTransform.getTranslateInstance(x,y);

            ((GeneralPath) s).transform(at);

            return s;



        } else {

            AffineTransform at1 = AffineTransform.getTranslateInstance(x,y);

            return(at1.createTransformedShape(s));

        }

    }

}

