/*

 * $Id: ManhattanConnector.java,v 1.1.1.1 2000/05/15 22:11:03 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 *

 */



package diva.canvas.connector;



import diva.canvas.Figure;

import diva.canvas.Site;

import diva.util.java2d.PaintedPath;

import diva.util.java2d.Polyline2D;



import java.awt.Graphics2D;

import java.awt.Paint;

import java.awt.Shape;

import java.awt.Stroke;

import java.awt.geom.AffineTransform;

import java.awt.geom.Line2D;

import java.awt.geom.Rectangle2D;



/** A Connector that draws itself with perpendicular lines. The connector

 * uses an instance of PaintedPath to draw itself, so see that class

 * for a more detailed description of the paint- and stroke-related

 * methods. To help it route itself, the connector contains an

 * instance of ManhattanRouter, which can be changed to create

 * other kinds of (or smarter) connectors.

 *

 * @version $Revision: 1.1.1.1 $

 * @author  John Reekie (johnr@eecs.berkeley.edu)

 * @author  Michael Shilman (michaels@eecs.berkeley.edu)

 */

public class ManhattanConnector extends AbstractConnector {



    /** The router used to route this connector.

     */

    private ManhattanRouter _router;



    /** Create a new manhattan connector between the given

     * sites. The connector is drawn with a width of one

     * and in black. The router is an instance of BasicManhattanRouter.

     * The connector is not routed until route() is called.

     */

    public ManhattanConnector (Site tail, Site head) {

        this(tail, head, new BasicManhattanRouter());

    }



    /** Create a new manhattan connector between the given sites and

     * with the given router. The connector is drawn with a width of one

     * and in black. The connector is routed between the head and tail

     * sites.

     */

    public ManhattanConnector (Site tail, Site head, ManhattanRouter router) {

        super(tail, head);

        setRouter(router);

    }



    /** Return the router.

     */

    public ManhattanRouter getRouter () {

        return _router;

    }



    /** Tell the connector to reposition the text label.

     * Not implemented yet.

     */

    public void repositionLabel () {

        throw new UnsupportedOperationException("Not implemented");

    }



    /** Tell the connector to route itself between the

     * current positions of the head and tail sites.

     */

    public void route () {

        repaint();

        getPaintedPath().shape =  _router.route(this);

        repaint();

    }



    /** Set the router and route again.

     */

    public void setRouter (ManhattanRouter router) {

        this._router = router;

        getPaintedPath().shape =  _router.route(this);

    }



    /** Translate the connector. This method is implemented, since

     * controllers may wish to translate connectors when the

     * sites at both ends are moved the same distance.

     */

    public void translate (double x, double y) {

        repaint();

        Polyline2D line = (Polyline2D) getPaintedPath().shape;

        line.translate(x,y);

        repaint();

    }

}

