/*

 * $Id: PerimeterSite.java,v 1.1.1.1 2000/05/15 22:11:03 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 *

 */



package diva.canvas.connector;



import diva.canvas.AbstractSite;

import diva.canvas.Site;

import diva.canvas.Figure;



import java.awt.Shape;



import java.awt.geom.Ellipse2D;

import java.awt.geom.Point2D;

import java.awt.geom.Rectangle2D;



/** A site that locates itself on the perimeter of a figure.

 * It implements the setNormal() method to change its position

 * so that a line drawn through the site at the given normal

 * will appear to originate from the center of the figure.

 * Currently, perimeter sites are able to locate themselves

 * on the perimeter of circles and rectangles.

 *

 * @version	$Revision: 1.1.1.1 $

 * @author 	John Reekie

 */

public class PerimeterSite extends AbstractSite {



    /** The id

     */

    private int _id;



    /** The normal

     */

    private double _normal;



    /** The parent figure

     */

    private Figure _parentFigure;



    /** Create a new site on the given figure with the given ID

     * and at the given normal.

     */

    public PerimeterSite (Figure figure, int id, double normal) {

        _parentFigure = figure;

        _id = id;

        setNormal(normal);

    }



    /** Get the figure to which this site is attached.

     */

    public Figure getFigure () {

        return _parentFigure;

    }



    /** Get the ID of this site.

     */

    public int getID () {

        return _id;

    }



    /** Get the point location of the site, in the enclosing

     * transform context.

     */

    public Point2D getPoint () {

        // We operate on the figure's shape, not the bounds!

        Shape shape = _parentFigure.getShape();

        Rectangle2D bounds = shape.getBounds2D();



        // Get the center of the parent

        double x = bounds.getX();

        double y = bounds.getY();

        double width = bounds.getWidth();

        double height = bounds.getHeight();



        double xCenter = x + width / 2;

        double yCenter = y + height / 2;



        // Switch on the shape of the figure

        double xout = 0.0, yout= 0.0;

        double pi = Math.PI;

        double pi2 = Math.PI / 2.0;

        double alpha = _normal;

        double beta = pi2 - alpha;



        if (shape instanceof Ellipse2D) {

            double rx = width / 2.0;

            double ry = height / 2.0;

            double dx, dy;



            if (Math.abs(width - height) < 0.001) {

                // Circle

                dx = rx * Math.cos(alpha);

                dy = ry * Math.sin(alpha);

            } else {

                // Ellipse

                if (Math.abs(alpha-pi2) < 0.001) {

                    dx = 0.0;

                    dy = ry;

                } else if (Math.abs(alpha+pi2) < 0.001) {

                    dx = 0.0;

                    dy = +ry;

                } else {

                    double m = Math.tan(alpha);

                    dx = rx * ry / (Math.sqrt(ry*ry + rx*rx * m*m));

                    if (alpha > pi2 || alpha < -pi2) {

                        dx = -dx;

                    }

                    dy = m * dx;

                }

            }

            xout = x + rx + dx;

            yout = y + ry + dy;

            

        } else {

	    if ( !(shape instanceof Rectangle2D)) {

		// Anything that's not a rectangle looks like one...

		shape = bounds;

	    }

            // The angle of the top-right corner

            double t = Math.atan2(height,width);

            if (alpha < -pi+t || alpha > pi - t) {

                // on left edge

                xout = x;

                yout = y + height/2.0 - width/2.0 * Math.tan(alpha);

            } else if (alpha < -t) {

                // top edge

                yout = y;

                xout = x + width/2.0 - height/2.0 * Math.tan(beta);

            } else if (alpha < t) {

                // right edge

                xout = x + width;

                yout = y + height/2.0 + width/2.0 * Math.tan(alpha);

            } else {

                // on bottom edge

                yout = y + height;

                xout = x + width/2.0 + height/2.0 * Math.tan(beta);

            }

        }

        return new Point2D.Double(xout, yout);

    } 



    /** Get the x-coordinate of the site. The site

     * is located in the center of the parent figure's bounding

     * box.

     */

    public double getX () {

        return getPoint().getX();

    }



    /** Get the y-coordinate of the site.  The site

     * is located in the center of the parent figure's bounding

     * box.

     */

    public double getY () {

        return getPoint().getY();

    }



    /** Set the normal "out" of the site. The site effectively

     * moves so that it passes through the center of the given figure.

     * The normal is limited to be between -pi and pi.

     */

    public void setNormal (double normal) {

        if (normal == _normal) {

            return;

        }

        while (normal > Math.PI) {

            normal -= 2 * Math.PI;

        }

        while (normal < -Math.PI) {

            normal += 2 * Math.PI;

        }

        _normal = normal;

    }

}



