/*

 * $Id: SelectionInteractor.java,v 1.1.1.1 2000/05/15 22:11:05 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 *

 */

package diva.canvas.selection;



import diva.canvas.Figure;

import diva.canvas.FigureDecorator;



import diva.canvas.event.LayerEvent;

import diva.canvas.event.LayerEventMulticaster;

import diva.canvas.event.LayerListener;

import diva.canvas.event.LayerMotionListener;

import diva.canvas.event.MouseFilter;



import diva.canvas.interactor.Interactor;

import diva.canvas.interactor.CompositeInteractor;

import diva.canvas.interactor.DragInteractor;



import diva.canvas.manipulator.Manipulator;



import java.awt.event.MouseEvent;

import java.util.ArrayList;

import java.util.Iterator;



/** 

 * A SelectionInteractor is attached to an object that can be put

 * into and out of a selection. Associated with each such role

 * is a selection model, which is the selection that figures

 * are added to or removed from.

 *

 * <p> When an item is selected, events are then forwarded to

 * other interactors. If, however, the clicked-on figure has

 * just been removed from the figure, do not forward events.

 *

 * @version $Revision: 1.1.1.1 $

 * @author John Reekie

 */

public class SelectionInteractor extends CompositeInteractor {



    /** Mode flag for delegation that indicates that a single

     * object is selected.

     */

    public final static int ONE_SELECTED = 4;



    /** Mode flag for delegation that indicates that many objects

     * are selected.

     */

    public final static int MANY_SELECTED = 8;



    /** Mode flag for delegation that indicates that the current

     * object has been newly selected.

     */

    public final static int NEWLY_SELECTED = 16;



    /** Mode flag for delegation that indicates that the current

     * object was already selected.

     */

    public final static int ALREADY_SELECTED = 32;



    /** Mode flags meaning that any selection is OK.

     */

    public final static int SELECTED = 64;



    /** The selection state

     */

    private int _state;



    /** The selection model

     */

    private SelectionModel _selection;



    /** The selection renderer

     */

    private SelectionRenderer _renderer;



    /** The mouse filter for selecting items

     */

    private MouseFilter _selectionFilter = MouseFilter.selectionFilter;



    /** The mouse filter for toggling items

     */

    private MouseFilter _toggleFilter = MouseFilter.alternateSelectionFilter;





    ///////////////////////////////////////////////////////////////////

    ////                         constructors                      ////



    /**

     * Create a new SelectionInteractor with a default selection model and

     * a default selection renderer.

     */

    public SelectionInteractor () {

        super();

        setSelectionRenderer(new BasicSelectionRenderer());

        setSelectionModel(new BasicSelectionModel());

    }



    /**

     * Create a new SelectionInteractor with the given selection model

     * and a null selection renderer.

     */

    public SelectionInteractor (SelectionModel model) {

        super();

        setSelectionRenderer(new BasicSelectionRenderer());

        setSelectionModel(model);

    }



    ///////////////////////////////////////////////////////////////////

    //// public methods



    /**

     * Accept an event if it will be accepted by the selection

     * filters.

     */

    public boolean accept (LayerEvent e) {

	return _selectionFilter.accept(e) || _toggleFilter.accept(e)

	    || super.accept(e);;

    }



     /**

     * Get the mouse filter that controls when this selection

     * filter is activated.

     */

    public MouseFilter getSelectionFilter () {

        return _selectionFilter;

    }



    /**

     * Get the selection model

     */

    public SelectionModel getSelectionModel () {

        return _selection;

    }



    /**

     * Get the selection renderer

     */

    public SelectionRenderer getSelectionRenderer () {

        return _renderer;

    }



    /**

     * Get the mouse filter that controls the toggling of

     * selections

     */

    public MouseFilter getToggleFilter () {

        return _toggleFilter;

    }



    /**

     * Handle a mouse press event. Figure out whether to

     * delegate events to another listener, and do it.

     * If noone here handles the event, pass it to the

     * superclass' method. The event is not consumed

     * by default.

     */

    public void mousePressed (LayerEvent event) {

        if (!isEnabled()) {

            return;

        }

        Figure figure = event.getFigureSource();

        int _state = 0;



        // Figure out the selection state

        if (_selection.getSelectionCount() == 1) {

            _state = ONE_SELECTED;

        } else if (_selection.getSelectionCount() > 1) {

            _state = MANY_SELECTED;

	}

	if (_state != 0) {

            if (_selection.containsSelection(figure)) {

                _state |= ALREADY_SELECTED;

            } else {

                _state |= NEWLY_SELECTED;

            }

	}



        boolean isSelected = false;

        if (_selectionFilter.accept(event)) {

            // If the item is not already in the selection, clear

            // the selection and then add this one.

            if ( !(_selection.containsSelection(figure))) {

		_selection.clearSelection();

		_selection.addSelection(figure);

            }

            isSelected = true;



        } else if (_toggleFilter.accept(event)) {

            // Toggle the item

            if (_selection.containsSelection(figure)) {

		_selection.removeSelection(figure);

            } else {

		_selection.addSelection(figure);

		isSelected = true;

            }

        }



        // Set the target of all attached drag interactors

        if (_selection.getSelectionCount() > 0 

                && _selection.containsSelection(figure)) {

            Object target[] = _selection.getSelectionAsArray();

            Iterator i = interactors();

            while (i.hasNext()) {

                Interactor interactor = (Interactor) i.next();

                if (interactor instanceof DragInteractor) {

                    ((DragInteractor) interactor).setTargetArray(target);

                }

            }

        }

 	// Allow superclass to process event

        super.mousePressed(event);

    }



    /**

     * Set the selection model. The existing selection model is

     * cleared first.

     */

    public void setSelectionModel (SelectionModel model) {

        if (_selection != null) {

            _selection.clearSelection();

        }

        _selection = model;

    }



    /**

     * Set the mouse filter that controls when this selection

     * filter is activated.

     */

    public void setSelectionFilter(MouseFilter f) {

        _selectionFilter = f;

    }



    /** Set the prototype decorator for selected figures.  Selected

     * figures will have a copy of this decorator wrapped around them.

     * This call is a convenience short-hand for

     * <pre>

     *    this.setSelectionRenderer(new BasicSelectionRenderer(decorator);

     * </pre>

     * <P>

     * This method nullifies any previous renderers set with

     * setSelectionRenderer();

     */

    public void setPrototypeDecorator (FigureDecorator decorator) {

        _renderer = new BasicSelectionRenderer(decorator);

    }



    /** Set the prototype selection manipulator. Selected figures

     * will have a copy of this manipulator wrapped around them.

     * This method nullifies any previous renderers set with

     * setSelectionRenderer();

     *

     * @deprecated Use setPrototypeDecorator instead

     */

    public void setSelectionManipulator (Manipulator manipulator) {

        _renderer = new BasicSelectionRenderer(manipulator);

    }



    /**

     * Set the selection renderer. The object is not used in this class,

     * but can be set here so that other objects that need to render

     * selected objects know how to do so.

     */

    public void setSelectionRenderer (SelectionRenderer r) {

        _renderer = r;

    }



    /**

     * Set the mouse filter that controls the toggling of

     * selections.

     */

    public void setToggleFilter(MouseFilter f) {

        _toggleFilter = f;

    }

}

