/*

 * $Id: PaintedShape.java,v 1.1.1.1 2000/05/15 22:11:05 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 *

 */



package diva.canvas.toolbox;



import diva.canvas.Figure;

import diva.canvas.AbstractFigure;

import diva.canvas.CanvasUtilities;

import diva.canvas.manipulator.ShapedFigure;



import java.awt.AlphaComposite;

import java.awt.BasicStroke;

import java.awt.Color;

import java.awt.Composite;

import java.awt.Graphics2D;

import java.awt.Paint;

import java.awt.Shape;

import java.awt.Stroke;

import java.awt.geom.AffineTransform;

import java.awt.geom.RectangularShape;

import java.awt.geom.Rectangle2D;



/** A utility class that paints a shape with an outline and/or fill.

 * This class is intended for use as a low-level class to simplify

 * construction of figures.  It contains a number of fields that

 * govern how the shape is painted, such as the fill paint, outline

 * stroke and/or dashing, outline paint, and alpha-composite.

 *

 * @deprecated Use diva.util.java2d.PaintedShape or

 * diva.util.java2d.PaintedPath instead

 *

 * @version	$Revision: 1.1.1.1 $

 * @author 	John Reekie

 */

public class PaintedShape {



     /** The dash array in the outline stroke.

     */

    private float _dashArray[] = null;



     /** The width of the outline stroke. -1 means that the line width is

     * set by the stroke. 0 means that the outline is not drawn.

     */

    private float _lineWidth = 1.0f;



    /** The stroke for the outline.

     */

    private Stroke _outlineStroke = null;



    /** The color compositing operator for the fill.

     */

    public Composite composite = AlphaComposite.SrcOver; // opaque



    /** The paint for the fill.

     */

    public Paint fillPaint = null;



    /** The shape of the figure.

     */

    public Shape shape = null;



    /** The paint for the outline.

     */

    public Paint outlinePaint = Color.black;



   /** Create a painted shape object with no Shape.

     */

    public PaintedShape () {

        shape = null;

    }



   /** Create a painted shape object on the given no Shape.

     */

    public PaintedShape (Shape s) {

        shape = s;

    }



   /** Get the bounding box of this figure. This method takes

     * account of the thickness of the stroke, if there is one.

     */

    public Rectangle2D getBounds () {

        if (_outlineStroke == null) {

            return shape.getBounds2D();

        } else {

            return _outlineStroke.createStrokedShape(shape).getBounds2D();

        }

    }



   /** Get the line width of this figure.

    */

    public float getLineWidth () {

      return _lineWidth;

    }



    /** Get the stroke of this figure.

     */

    public Stroke getStroke () {

        return _outlineStroke;

    }



    /** Test if this shape intersects the given rectangle. Currently

     * this does not take into account the width of the stroke

     * or other things such as dashes, because of problems with

     * geometry testing with GeneralPath in the first version of

     * JDK1.2.

     */

    public boolean hit (Rectangle2D r) {

        // Hit testing on strokes doesn't appear to work too

        // well in JDK1.2, so we will cheat and ignore the width

        // of the stroke

        if (fillPaint != null) {

            // There's a fill, so test the internal of the path

            return shape.intersects(r);

        } else if (outlinePaint != null && _outlineStroke != null) {

            // Otherwise test for intersection with the path

            return shape.intersects(r);

        } else {

            return false;

        }

    }



    /** Paint the shape. The shape is redrawn with the current

     *  shape, fill, and outline.

     */

    public void paint (Graphics2D g) {

        // Fill it

        if (fillPaint != null && composite != null) {

            g.setPaint(fillPaint);

            g.setComposite(composite);

            g.fill(shape);

        }

        // Stroke it

        if (_outlineStroke != null && outlinePaint != null) {

            g.setStroke(_outlineStroke);

            g.setPaint(outlinePaint);

            g.setComposite(AlphaComposite.SrcOver); // opaque

            g.draw(shape);

        }

    }



   /** Set the dash array of this figure. If the current stroke

    * is an instance of BasicStroke, then its dash array will be

    * set to the passed array; otherwise, the old stroke will be

    * deleted and replaced with a new one with the current line

    * width.

    */

    public void setDashArray (float dashArray[]) {

      this._dashArray = dashArray;

      if (_outlineStroke instanceof BasicStroke) {

	  BasicStroke s = (BasicStroke) _outlineStroke;

	  _outlineStroke = new BasicStroke(

		    _lineWidth,

		    s.getEndCap(),

		    s.getLineJoin(),

		    s.getMiterLimit(),

		    _dashArray,

		    0.0f);

      } else {

	_outlineStroke = new BasicStroke(

		    _lineWidth,

		    BasicStroke.CAP_SQUARE,

		    BasicStroke.JOIN_MITER,

		    10.0f,

		    _dashArray,

		    0.0f);

      }

    }



   /** Set the line width of this figure. If the current stroke

    * is an instance of BasicStroke, them its width will be

    * set to the passed width; otherwise, the old stroke will be

    * deleted and replaced with a new one with the given

    * line width. If the width is zero, then the stroke will

    * be removed.

    */

    public void setLineWidth (float lineWidth) {

      _lineWidth = lineWidth;

      if (_outlineStroke instanceof BasicStroke) {

	BasicStroke s = (BasicStroke) _outlineStroke;

	_outlineStroke = new BasicStroke(

		    lineWidth,

		    s.getEndCap(),

		    s.getLineJoin(),

		    s.getMiterLimit(),

		    _dashArray,

		    0.0f);

      } else {

	_outlineStroke = new BasicStroke(

		    _lineWidth,

		    BasicStroke.CAP_SQUARE,

		    BasicStroke.JOIN_MITER,

		    10.0f,

		    _dashArray,

		    0.0f);

      }

    }



    /** Set the stroke of this figure.

     */

    public void setStroke (Stroke s) {

        _outlineStroke = s;

	if (s instanceof BasicStroke) {

	  _lineWidth = ((BasicStroke)s).getLineWidth();

	  _dashArray = ((BasicStroke)s).getDashArray();

	} else {

	  _lineWidth = 1.0f;

	  _dashArray = null;

	}

    }

}

