/*
 * $Id: GraphView.java,v 1.1.1.1 2000/05/15 22:11:06 dauplait Exp $
 *
 * Copyright (c) 1998 The Regents of the University of California.

 */
   package diva.graph;

   import diva.graph.layout.*;
   import diva.graph.model.*;
   import diva.canvas.*;
   import diva.canvas.event.*;
   import diva.canvas.selection.*;
   import diva.canvas.toolbox.*;
   import java.awt.event.*;
   import java.awt.geom.*;
   import java.util.*;
   import diva.util.*;
   import diva.canvas.connector.Connector;
   import diva.canvas.connector.AbstractConnector;

   import carto.graphe.Noeud;

/**
 * GraphView is the part of the GraphPane widget which controls the display
 * of graph elements.
 *
 * @author 	Michael Shilman (michaels@eecs.berkeley.edu)
 * @version	$Revision: 1.1.1.1 $
 * @rating      Red
 */
   public class GraphView implements LayoutTarget {
    /**
     * The hashtable that stores the mapping from
     * nodes/edges to the figures that view them.
     */
      protected HashMap _viewMapping = new HashMap();
   
    /**
     * The model that is being displayed.
     */
      private GraphModel _model;
   
    /**
     * The edge renderer for this view.
     */
      private EdgeRenderer _edgeRenderer = new BasicEdgeRenderer();
   
    /**
     * The node renderer for this view.
     */
      private NodeRenderer _nodeRenderer = new BasicNodeRenderer();
   
    /**
     * The controller of this view.
     */
      private GraphController _controller;
   
    /**
     * The graphics pane that this is controlling.
     */
      private GraphicsPane _pane;
   
    /**
     * The percentage of the screen that should be used
     * for layout.
     */
      private double _layoutPercentage = .8;
   
    /**
     * The incremental layout engine for this view.
     */
      private IncrementalLayout _incrLayout
      = new diva.graph.layout.RandomIncrLayout(this);
   
    /**
     * Construct a graph view. The graph cannot be used until
     * it has a pane set with setGraphPane(), which is
     * called when the view is added to the pane.
     */
      public GraphView () {
        //
      }
   
   /**
     * Add an edge figure to the pane.  Assume
     * that the mapping in both directions has
     * already been set up.
     */
      public void addEdgeFigureToLayer(Figure f) {
         _pane.getForegroundLayer().add(f);
      
        // Raise the figure to the top -- FIXME: conditional?
         ZList zl = _pane.getForegroundLayer().getFigures();
         zl.setIndex(0, f);
      
         f.repaint();
      }
   
   /**
     * Set up the mapping between an edge and its corresponding
     * figure. The figure's user object is set to point to the
     * edge, and an entry mapping from the edge to the figure is
     * added to the internal hash table, if it doesn't already
     * contain that mapping. If the edge is already in the hash
     * table but it maps to a different figure, throw an
     * exception.
     */
      public void addEdgeMapping(Edge e, Connector ef) {
         if (_viewMapping.containsKey(e)) {
            if (_viewMapping.get(e) != ef) {
               throw new IllegalStateException(
                                    "Edge " + e + "does not map to figure " + ef);
            }
            ef.setUserObject(e);
         } 
         else { 
            ef.setUserObject(e);
            _viewMapping.put(e, ef);
         }
      }
   
    /**
     * Add a node figure to the pane.  Assume
     * that the mapping in both directions has
     * already been set up.
     */
      public void addNodeFigureToLayer(Figure nf) {
         _pane.getForegroundLayer().add(nf);
      }
   
    /**
     * Add a node figure to its logical parent.  Assume
     * that the mapping in both directions has
     * already been set up.
     */
      protected void addNodeFigureToParent(Figure nf) {
         Node n = (Node)nf.getUserObject();
         CompositeNode cn = (CompositeNode)n.getParent();
         CompositeFigure cnf = (CompositeFigure)getNodeFigure(cn);
         cnf.add(nf);
      
        //FIXME: translate to preserve the position on the screen?
         try {
            nf.transform(cnf.getTransformContext().getTransform().createInverse());
         }
            catch(Exception e) {
               debug("error: " + e);
               System.exit(-1);
            }
      }
   
   /**
     * Set up the mapping between a node and its corresponding
     * figure. The figure's user object is set to point to the
     * node, and an entry mapping from the node to the figure is
     * added to the internal hash table, if it doesn't already
     * contain that mapping. If the node is already in the hash
     * table but it maps to a different figure, throw an
     * exception.
     */
      public void addNodeMapping(Node n, Figure nf) {
         if (_viewMapping.containsKey(n)) {
            if (_viewMapping.get(n) != nf) {
               throw new IllegalStateException(
                                    "Node " + n + "does not map to figure " + nf);
            }
            nf.setUserObject(n);
         } 
         else { 
            nf.setUserObject(n);
            _viewMapping.put(n, nf);
         }
      }
   
    /**
     * Clear all figures from the display
     */
      public void clear () {
         for(Iterator i = _viewMapping.values().iterator(); i.hasNext(); ) {
            Figure f = (Figure)i.next();
            _pane.getForegroundLayer().remove(f);
         }
         _viewMapping.clear();
      }
   
    /**
     * Debugging output.
     */
      private void debug(String s) {
         System.err.println("GraphView: " + s);
      }
   
    /**
     * Return the bounds of the figure associated with the given node
     * in the target's view.
     *
     * @see LayoutTarget.getBounds(Node)
     */
      public Rectangle2D getBounds(Node n) {
         Figure f = getNodeFigure(n);
         return f.getBounds();
      }
   
    /**
     * Return the visual representation of the given edge as a
     * Connector.
     */
      public Connector getEdgeFigure(Edge e) {
         return (Connector)_viewMapping.get(e);
      }
   
    /**
     * Return the edge renderer for this view.
     */
      public EdgeRenderer getEdgeRenderer() {
         return _edgeRenderer;
      }
   
    /**
     * Return the graph controller.
     */
      public GraphController getGraphController() {
         return _controller;
      }
   
    /**
     * Return the graph model being viewed.
     */
      public GraphModel getGraphModel() {
         return _model;
      }
   
    /**
     * Return the graphics pane of this controller.
     */
      public GraphicsPane getGraphicsPane() {
         return _pane;
      }
   
    /**
     * Return the incremental graph layout algorithm that gets called
     * whenever the graph topology is changed.
     */
      public IncrementalLayout getIncrLayout() {
         return _incrLayout;
      }
   
    /**
     * Return the percentage of the pane that is used for layout.
     *
     * @see setLayoutPercentage(double)
     */
      public double getLayoutPercentage() {
         return _layoutPercentage;
      }
   
    /**
     * Return the visual representation of the given node as a
     * Figure.
     */
      public Figure getNodeFigure(Node n) {
         return (Figure)_viewMapping.get(n);
      }
   
    /**
     * Return the node renderer for this view.
     *
     * @see setNodeRenderer(NodeRenderer)
     */
      public NodeRenderer getNodeRenderer() {
         return _nodeRenderer;
      }
   
    /**
     * Return the viewport for layout.
     *
     * @see LayoutTarget.getViewport()
     */
      public Rectangle2D getViewport(Graph g) {
         if(g == _model.getGraph()) {
            Point2D p = _pane.getSize();
         
            double borderPercentage = (1-getLayoutPercentage())/2;
            double x = borderPercentage*p.getX();
            double y = borderPercentage*p.getY();
            double w = getLayoutPercentage()*p.getX();
            double h = getLayoutPercentage()*p.getY();
            return new Rectangle2D.Double(x, y, w, h);
         }
         else if(g instanceof CompositeNode) {
            CompositeNode n = (CompositeNode)g;
            //XXX cannot do away with CNF?
            CompositeNodeFigure f = (CompositeNodeFigure)getNodeFigure(n);
            if(f != null) {
               Rectangle2D r = f.getBackgroundFigure().getBounds();
               return new Rectangle2D.Double(0, 0,
                                             r.getWidth(), r.getHeight());
            }
         }
         throw new IllegalArgumentException(
                              "Unknown graph.  Cannot determine viewport.");
      }
   
    /**
     * Return an iterator over the nodes which intersect the given
     * rectangle in the top-level graph or any of its sub-graphs
     * recursively.
     */
      public Iterator recursivelyIntersectingNodes(Rectangle2D r) {
         ZList zl = _pane.getForegroundLayer().getFigures();
         ArrayList results = new ArrayList();
         recursiveIntersectSub(results, zl, r);
         return results.iterator();
      }
   
    /**
     * Recursively search through the Z list looking for intersecting
     * figures that represent nodes, and placing the nodes in the list
     * of results, to be returned by the recursivelyIntersectingNodes()
     */
      private void recursiveIntersectSub(List results, ZList zl, Rectangle2D r) {
         Iterator i = zl.getIntersectedFigures(r).figuresFromFront();
         while(i.hasNext()) {
            Figure f = (Figure)i.next();
            Object o = f.getUserObject();
            if(o instanceof Node) {
               results.add(o);
            }
            if(f instanceof CompositeFigure) {
               CompositeFigure cf = (CompositeFigure)f;
                //transform a copy of r into the child's coordinate frame
            
               AffineTransform at = cf.getTransformContext().getTransform();
               try {
                  at = at.createInverse();
               }
                  catch(NoninvertibleTransformException ex) {
                     throw new RuntimeException(ex.getMessage());
                  }
               Rectangle2D r2 = (Rectangle2D) r.clone();
               r2 = CanvasUtilities.transform(r2, at).getBounds();
               recursiveIntersectSub(results, cf.getChildren(), r2);
            }
         }
      }
   
   
    /**
     * Return an iterator over the nodes which intersect the given
     * rectangle in the top-level graph.
     */
      public Iterator intersectingNodes(Rectangle2D r) {
         ZList zlist = _pane.getForegroundLayer().getFigures();
         Iterator i = zlist.getIntersectedFigures(r).figuresFromFront();
         Iterator j = new FilteredIterator(i, 
                                              new Filter() {
                                                 public boolean accept(Object o) {
                                                    Figure f = (Figure)o;
                                                    return (f.getUserObject() instanceof Node);
                                                 }
                                              });
      
         return 
               new ProxyIterator(j) {
                  public Object next() {
                     Figure nf = (Figure)super.next();
                     return (Node)nf.getUserObject();
                  }
               };
      }
   
    /**
     * Return an iterator over the node or edge figures which
     * intersect the given rectangle.
     */
      public Iterator intersectingEdges(Rectangle2D r) {
         ZList zlist = _pane.getForegroundLayer().getFigures();
         Iterator i = zlist.getIntersectedFigures(r).figuresFromFront();
         Iterator j = new FilteredIterator(i, 
                                              new Filter() {
                                                 public boolean accept(Object o) {
                                                    return (o instanceof Connector);
                                                 }
                                              });
         return 
               new ProxyIterator(j) {
                  public Object next() {
                     Connector ef = (Connector)super.next();
                     return (Edge)ef.getUserObject();
                  }
               };
      }
   
    /**
     * Return whether or not the given node is actually
     * visible in the view.
     *
     * @see LayoutTarget.isNodeVisible(Node)
     */
      public boolean isNodeVisible(Node n) {
         Figure nf = getNodeFigure(n);
         return (nf != null && nf.isVisible() && nf.getParent() != null);
      }
   
    /**
     * Return whether or not the given edge is actually
     * visible in the view.
     *
     * @see LayoutTarget.isEdgeVisible(Edge)
     */
      public boolean isEdgeVisible(Edge e) {
         Connector ef = getEdgeFigure(e);
         return (ef != null && ef.isVisible() && ef.getParent() != null);
      }
   
    /**
     * Given a node figure that has already been added to the display,
     * relocate it so that its top-left corner is in the given (x,y)
     * location.
     */
      public void placeNodeFigure(Figure nf, double x, double y) {
         Rectangle2D bounds = nf.getBounds();
         double xdash = x - bounds.getWidth() / 2;
         double ydash = y - bounds.getHeight() / 2;
         nf.translate(xdash - bounds.getX(), ydash - bounds.getY());
      }
   
    /**
     * Remove an edge figure's edge from 
     * the model (and the figure will be removed
     * accordingly).
     *
     * @deprecated
     */
      public void removeEdgeFigure(Connector ef) {
         _model.disconnectEdge((Edge)ef.getUserObject());
      }
   
    /**
     * Remove the mapping between an edge and its corresponding
     * figure. The edge is removed from the internal hash table and
     * the figure's user object is set to null. Throw an exception
     * if the edge is not in the mapping.
     */
      public void removeEdgeMapping(Edge e, Figure ef) {
         if (!_viewMapping.containsKey(e)) {
            throw new IllegalStateException(
                                 "Edge " + e + "does not map to any figure");
         }
         if (_viewMapping.get(e) != ef) {
            throw new IllegalArgumentException(
                                 "Edge " + e + "does not map to figure " + ef);
         }
         
         // le lien ne disparait pas dans l'editeur, alors on le deplace
         //@author Claudia
         ef.setVisible(false);
         ef.translate(1000, 1000);
         if ( ef instanceof AbstractConnector) {
         	((AbstractConnector)ef).getHeadEnd().translate(1000, 1000);
         	}
         	
         ef.setUserObject(null);
         _viewMapping.remove(e);
         
         // actualiser le model 
         //@author Claudia
         ((BasicNode)e.getTail()).getOutEdgeSet().remove(e);
         ((BasicNode)e.getHead()).getInEdgeSet().remove(e);
      }
   
    /**
     * Remove a node figure's node from 
     * the model (and the figure will be removed
     * accordingly).
     *
     * @deprecated
     */
      public void removeNodeFigure(Figure nf) {
         GraphUtilities.purgeNode(_model.getGraphImpl(),(Node)nf.getUserObject());
      }
   
    /**
     * Remove the mapping between a node and its corresponding
     * figure. The node is removed from the internal hash table and
     * the figure's user object is set to null. Throw an exception
     * if the node is not in the mapping.
     */
      public void removeNodeMapping(Node n, Figure nf) {
         if (!_viewMapping.containsKey(n)) {
            throw new IllegalStateException(
                                 "Node " + n + "does not map to any figure");
         }
         if (_viewMapping.get(n) != nf) {
            throw new IllegalArgumentException(
                                 "Node " + n + "does not map to figure " + nf);
         }
         nf.setUserObject(null);
      
        //placeNodeFigure => la figure du noeud ne disparait pas dans le GraphPane
        //@author Claudia
         placeNodeFigure(nf, 1000, 1000);
      
         _viewMapping.remove(n);
      }
    /**
     * Remplace n1 par n2 dans le _viewMapping.
     * @author CLaudia
     */	
      public void changeNodeMapping(Node n1, Node n2) {
         if (!_viewMapping.containsKey(n1)) 
            throw new IllegalStateException(
                                 "Node " + n1 + "does not map to any figure");
         if ( _viewMapping.get(n1) instanceof LabelWrapper) {
            ((LabelWrapper)_viewMapping.get(n1)).getLabel().setString(((Noeud)n2).getIdent());
            GraphEvent e = new GraphEvent(GraphEvent.STRUCTURE_CHANGED,this,_model.getGraph());
            getGraphicsPane().repaint();
         }
      	//changer les liens
         if ( n1 instanceof BasicNode && n2 instanceof BasicNode) {
            ((BasicNode)n2).setInEdgeSet(((BasicNode)n1).getInEdgeSet());
            ((BasicNode)n2).setOutEdgeSet(((BasicNode)n1).getOutEdgeSet());
         }
         addNodeMapping(n2,  (Figure)_viewMapping.get(n1));
         _viewMapping.remove(n1);
      }
   
    /**
     * Route absolutely the figure associated with the given edge in
     * the target's view.
     */
      public void route(Edge e) {
         Connector ef = getEdgeFigure(e);
         ef.route();
      }
   
    /**
     * Reroute the figure associated with the given edge in the
     * target's view given an incremental change in one of its
     * endpoints.
     *
    public void reroute(Edge e);
    */
   
    /**
     * Set the graph controller.
     */
      public void setGraphController(GraphController c) {
         _controller = c;
      }
   
    /**
     * Set the graph model
     */
      public void setGraphModel (GraphModel model) {
         _model = model;
      }
   
    /**
     * Set the graph pane. This is called by the GraphPane.
     */
      public final void setGraphPane (GraphPane pane) {
         _pane = pane;
      }
   
    /**
     * Set the edge renderer for this view.
     */
      public void setEdgeRenderer(EdgeRenderer er) {
         _edgeRenderer = er;
      }
   
    /**
     * Set the node renderer for this view.
     */
      public void setNodeRenderer(NodeRenderer nr) {
         _nodeRenderer = nr;
      }
   
    /**
     * Set the incremental graph layout algorithm that gets called
     * whenever the graph topology is changed.
     */
      public void setIncrLayout(IncrementalLayout l) {
         _incrLayout = l;
      }
   
    /**
     * Set the percentage of the pane that should be used for layout.
     * This method defines a frame 
     */
      public void setLayoutPercentage(double d) {
         if((d <= 0) || d > 1) {
            String err = "Layout percentage must be between 0 and 1";
            throw new IllegalArgumentException(err);
         }
         else {
            _layoutPercentage = d;
         }
      }
   
    /**
     * Translate the figure associated with the given node in the
     * target's view by the given delta.
     */
      public void translate(Node n, double dx, double dy) {
         Figure nf = getNodeFigure(n);
         nf.translate(dx, dy);
      }
   }
