/*

 * $Id: BubbleController.java,v 1.1.1.1 2000/05/15 22:11:06 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */

package diva.graph.bubble;



import diva.graph.GraphView;

import diva.graph.GraphPane;

import diva.graph.GraphController;

import diva.graph.AbstractGraphController;

import diva.graph.NodeInteractor;

import diva.graph.EdgeInteractor;

import diva.graph.NodeRenderer;

import diva.graph.EdgeRenderer;



import diva.graph.model.GraphModel;

import diva.graph.model.Graph;

import diva.graph.model.Node;

import diva.graph.model.Edge;



import diva.canvas.Figure;

import diva.canvas.FigureLayer;

import diva.canvas.GraphicsPane;

import diva.canvas.Site;



import diva.canvas.connector.AutonomousSite;

import diva.canvas.connector.CenterSite;

import diva.canvas.connector.PerimeterSite;

import diva.canvas.connector.PerimeterTarget;

import diva.canvas.connector.Connector;

import diva.canvas.connector.ArcConnector;

import diva.canvas.connector.ArcManipulator;

import diva.canvas.connector.ConnectorEvent;

import diva.canvas.connector.ConnectorListener;

import diva.canvas.connector.ConnectorTarget;



import diva.canvas.event.LayerAdapter;

import diva.canvas.event.LayerEvent;

import diva.canvas.event.MouseFilter;



import diva.canvas.interactor.Interactor;

import diva.canvas.interactor.AbstractInteractor;

import diva.canvas.interactor.DragInteractor;



import diva.canvas.manipulator.GrabHandle;

import diva.canvas.manipulator.Manipulator;

import diva.canvas.manipulator.BoundsManipulator;



import diva.canvas.selection.SelectionInteractor;

import diva.canvas.selection.SelectionModel;

import diva.canvas.selection.SelectionDragger;



import java.awt.event.InputEvent;

import java.util.HashMap;

import java.util.Iterator;



/**

 * A controller for bubble-and-arc graph editors.

 *

 * @author 	Michael Shilman (michaels@eecs.berkeley.edu)

 * @version	$Revision: 1.1.1.1 $

 * @rating      Red

 */

public class BubbleController extends AbstractGraphController {

 

    /** The selection interactor for drag-selecting nodes

     */

    private SelectionDragger _selectionDragger;



   /** The interactor for creating new nodes

     */

    private NodeCreator _nodeCreator;



   /** The interactor that interactively creates edges

     */

    private EdgeCreator _edgeCreator;



    /** The filter for control operations

     */

    private MouseFilter _controlFilter = new MouseFilter (

            InputEvent.BUTTON1_MASK,

            InputEvent.CTRL_MASK);



    /**

     * Create a new basic controller with default node and edge interactors.

     */

    public BubbleController () {



        // The interactors attached to nodes and edges

        SelectionModel sm = getSelectionModel();

        NodeInteractor ni = new NodeInteractor(this, sm);

        EdgeInteractor ei = new EdgeInteractor(this, sm);

        setNodeInteractor(ni);

        setEdgeInteractor(ei);



        PerimeterTarget ct = new PerimeterTarget() {

	    public boolean accept (Figure f) {

                return (f.getUserObject() instanceof Node);

            }

	};

        setConnectorTarget(ct);



        // Create and set up the manipulator for connectors

        ArcManipulator manipulator = new ArcManipulator();

        manipulator.setSnapHalo(4.0);

        manipulator.setConnectorTarget(ct);

        manipulator.addConnectorListener(new EdgeDropper());

        ei.setSelectionManipulator(manipulator);



        // The mouse filter needs to accept regular click or control click

        MouseFilter handleFilter = new MouseFilter(1, 0, 0);

        manipulator.setHandleFilter(handleFilter);

    }



    /**

     * Initialize all interaction on the graph pane. By the time

     * this method is called, all relevant references to views,

     * panes, and roles must already have been set up.

     */

    public void initializeInteraction () {

        GraphView view = getGraphView();

        GraphicsPane pane = view.getGraphicsPane();



        // Create and set up the selection dragger

        SelectionDragger _selectionDragger = new SelectionDragger(pane);

        _selectionDragger.addSelectionInteractor(getEdgeInteractor());

        _selectionDragger.addSelectionInteractor(getNodeInteractor());



        // Make it so that nodes get resize handles

        BoundsManipulator nodeManipulator = new BoundsManipulator();

        getNodeInteractor().setSelectionManipulator(nodeManipulator);



        // When the nodes are resized, reroute its attached edges

        DragInteractor dragger = nodeManipulator.getHandleInteractor();

        dragger.addLayerListener(new EdgeRerouter());



        // Create a listener that creates new nodes

        _nodeCreator = new NodeCreator();

        _nodeCreator.setMouseFilter(_controlFilter);

        pane.getBackgroundEventLayer().addInteractor(_nodeCreator);



        // Create the interactor that drags new edges.

        _edgeCreator = new EdgeCreator();

        _edgeCreator.setMouseFilter(_controlFilter);

        getNodeInteractor().addInteractor(_edgeCreator);

    }



    ///////////////////////////////////////////////////////////////

    //// EdgeRerouter



    /** An inner class reroutes edges attached to a node

     * when it is (presumably) reshaped.

     */

    protected class EdgeRerouter extends LayerAdapter {

        public void mouseDragged (LayerEvent e) {

            // The figure in the event is the grab-handle. Get the

            // actual node figure

            Manipulator m = (Manipulator) e.getFigureSource().getParent();

            Figure f = m.getChild();

            Node n = (Node) f.getUserObject();

           

            GraphView view = getGraphView();

            GraphicsPane pane = view.getGraphicsPane();



            // Reroute attached edges.

            // See NodeDragInteractor for more info.

            for (Iterator ins = n.inEdges(); ins.hasNext(); ) {

                Connector c = view.getEdgeFigure((Edge) ins.next());

                if (c != null) {

                    c.reroute();

                }

            }

            for (Iterator outs = n.outEdges(); outs.hasNext(); ) {

                Connector c = view.getEdgeFigure((Edge) outs.next());

                if (c != null) {

                    c.reroute();

                }

            }

        }

    }



    ///////////////////////////////////////////////////////////////

    //// NodeCreator



    /** An inner class that places a node at the clicked-on point

     * on the screen, if control-clicked with mouse button 1. This

     * needs to be made more customizable.

     */

    protected class NodeCreator extends AbstractInteractor {

        public void mousePressed(LayerEvent e) {

            GraphView view = getGraphView();

            GraphModel model = getGraphModel();



            // Create a new node

            Node n = model.createNode();



            // Create a figure for it

            Figure nf = view.getNodeRenderer().render(n);

            nf.setInteractor(getNodeInteractor());

 

            // Add to the view and model

            view.addNodeFigureToLayer(nf);

            view.placeNodeFigure(nf, e.getLayerX(), e.getLayerY());

            view.addNodeMapping(n, nf);

            model.addNode(n);

        }

    }



    ///////////////////////////////////////////////////////////////

    //// EdgeDropper



    /** An inner class that handles interactive changes to connectivity.

     */

    protected class EdgeDropper implements ConnectorListener {

        /**

         * Do nothing.

         */

        public void connectorDragged(ConnectorEvent evt) {}



        /**

         * Called when a connector end is dropped--attach or

         * detach the edge as appropriate.

         */

        public void connectorDropped(ConnectorEvent evt) {

            Connector c = evt.getConnector();

            Figure f = evt.getTarget();

            Edge e = (Edge)c.getUserObject();

            Node n = (f == null) ? null : (Node)f.getUserObject();

            GraphModel model = getGraphModel();

            switch (evt.getEnd()) {

            case ConnectorEvent.HEAD_END:

                model.setEdgeHead(e, n);

                break;

            case ConnectorEvent.TAIL_END:

                model.setEdgeTail(e, n);

                break;

            default:

                throw new IllegalStateException(

                        "Cannot handle both ends of an edge being dragged.");

            }

        }

    

        /**

         * Do nothing.

         */

        public void connectorSnapped(ConnectorEvent evt) {}



        /**

         * Do nothing.

         */

        public void connectorUnsnapped(ConnectorEvent evt) {}

    }



    ///////////////////////////////////////////////////////////////

    //// EdgeCreator



    /** An interactor that interactively drags edges from one node

     * to another.

     */

    protected class EdgeCreator extends AbstractInteractor {

        public void mousePressed(LayerEvent e) {

            GraphView view = getGraphView();

            GraphModel model = getGraphModel();



            Figure source = e.getFigureSource();

            FigureLayer layer = (FigureLayer) e.getLayerSource();

            double x = e.getLayerX();

            double y = e.getLayerY();



            // Create a new edge

            Edge edge = model.createEdge();



            // Create a figure for it

            Site tailSite = getConnectorTarget().getTailSite(source, x, y);

            Site headSite = new AutonomousSite(layer, x, y);

            Connector ef = view.getEdgeRenderer().render(edge, tailSite, headSite);

            ef.setInteractor(getEdgeInteractor());



            // Add to the view and model

            view.addEdgeFigureToLayer(ef);

            view.addEdgeMapping(edge, ef);

            model.setEdgeTail(edge, (Node) source.getUserObject());



            // Add it to the selection so it gets a manipulator, and

            // make events go to the grab-handle under the mouse

            getSelectionModel().addSelection(ef);

            ArcManipulator cm = (ArcManipulator) ef.getParent();

            GrabHandle gh = cm.getHeadHandle();

            layer.grabPointer(e, gh);

        }

    }

}

