/*

 * $Id: GridAnnealingLayout.java,v 1.1.1.1 2000/05/15 22:11:07 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */

package diva.graph.layout;

import diva.graph.model.*;



import java.awt.geom.Rectangle2D;

import java.awt.geom.Line2D;

import java.util.Random;

import java.util.HashMap;

import java.util.Iterator;



/**

 * A simple layout which places nodes on a grid using a cost function.

 * It performs the following simple-minded algorithm:

 *

 * <ol>

 *

 * <li> defines a grid based on the number of nodes in the graph and

 *   randomly assigns the nodes to vertices on the grid.

 *

 * <li> randomly swaps nodes on the grid and picks a good settling

 *  point based on a cost function which favors short edges over

 *  long ones, straight edges over diagonal ones, and generally

 *  tries not to put edges through nodes.

 *

 * <li> distorts the grid based on the relative sizes of the nodes,

 *   and finally places the nodes according to this distortion.  *

 *

 * </ol>

 *

 * This class is implemented as a large template method, so it should

 * be relatively easy to extend or modify.

 *

 * @author Michael Shilman (michaels@eecs.berkeley.edu)

 * @version $Revision: 1.1.1.1 $

 * @rating Red

 */

public class GridAnnealingLayout implements GlobalLayout {

    /**

     * The penalty for each unit of height in an edge.

     */

    private static final double HEIGHT_FACTOR = 1;



    /**

     * The penalty for each unit of width in an edge.

     */

    private static final double WIDTH_FACTOR = 1;



    /**

     * The penalty for elbow-shaped edges, i.e ones

     * which have height and width.

     */

    private static final double ELBOW_PENALTY = 10;



    /**

     * The penalty for overlaps with other edges in the graph.

     */

    private static final double EDGE_OVERLAP_PENALTY = 30;



    /**

     * The penalty for tees, where a node lands ON another

     * edge that is not connected to it.

     */

    private static final double TEE_PENALTY = 30;



    /**

     * The penalty for crossings with other edges in the graph.

     */

    private static final double CROSSING_PENALTY = 10;

    

    /**

     * The random number generator used in choosing which

     * nodes to swap.

     */

    protected Random _random = new Random(System.currentTimeMillis());

    

    /**

     * The layout target that is passed in by the user and

     * used to assign the layout to the view.

     */

    protected LayoutTarget _target;

    

    /**

     * The original graph that is passed in by the user on

     * which the layout is applied.

     */

    protected Graph _graph;



    /**

     * The grid width.

     */

    protected int _gw;



    /**

     * The grid height.

     */

    protected int _gh;



    /**

     * The current grid configuration as the algorithm progresses.

     */

    protected Node[][] _grid;



    /**

     * The best grid configuration so far as the algorithm progresses.

     */

    protected Node[][] _minGrid;



    /**

     * The relative cost of the best grid configuration so far as the

     * algorithm progresses.

     */

    protected double _minCost;



    /**

     * A mapping from nodes to their corresponding logical grid

     * positions, stored as integer arrays of length 2.

     */

    protected HashMap _map;



    /**

     * A sparseness measure for the layout.

     *

     * @see setSparseness(double)

     */

    protected double _sparseness = 1.0;



    /**

     * The number of iterations to cool over.

     */

    protected int _numIters = 5;



    /**

     * The number of moves per iteration.

     */

    protected int _numMoves = 10;



    /**

     * The cooling constant.

     */

    protected double _cool = .95;



    /**

     * Perform simulated annealing using the cost function.

     */

    private void anneal() {

        double curCost = _minCost = 0;

        double prob = 1.0;

        snapMin();



        for(int i = 0; i < _numIters; i++) {

            for(int j = 0; j < _numMoves; j++) {

                prob *= _cool;

                

                int x1 = (int)(_random.nextDouble()*_gw);

                int y1 = (int)(_random.nextDouble()*_gh);

                int x2 = (int)(_random.nextDouble()*_gw);

                int y2 = (int)(_random.nextDouble()*_gh);

                Node n1 = _grid[x1][y1];

                Node n2 = _grid[x2][y2];



                if(n1 != n2) {

                    double startCost = nodeCost(n1) + nodeCost(n2);

                    if(n1 == null) {

                        setXY(n2, x1, y1);

                    }

                    else if(n2 == null) {

                        setXY(n1, x2, y2);

                    }

                    else {

                        swap(n1, n2);

                    }



                    double endCost = nodeCost(n1) + nodeCost(n2);

                    double deltaCost = endCost-startCost;

                    curCost += deltaCost;

                

                    //   debug(">>> DELTA: " + deltaCost);



                    if(curCost < _minCost) {

                        _minCost = curCost;

                        snapMin();

                    }



                    // see if we should put the nodes back

                    // in their previous positions

                    if(deltaCost > 0 && _random.nextDouble() > prob) {

                        curCost -= deltaCost;

                        if(n1 == null) {

                            setXY(n2, x2, y2);

                        }

                        else if(n2 == null) {

                            setXY(n1, x1, y1);

                        }

                        else {

                            swap(n1, n2);

                        }

                    }

                }

            }

        }

        //  debug(" MIN_COST = " + _minCost + " ======================");

    }



    /**

     * Assert the given condition and throw a runtime exception with

     * the given string if the assertion fails.

     */

    private final void ASSERT(boolean b, String err) throws RuntimeException {

        if(!b) { throw new RuntimeException(err); }

    }



    /**

     * Assign layout positions to the nodes in the graph based on the

     * logical grid positions determined in the annealing process.

     */

    private void assignLayout() {

        Rectangle2D dim = _target.getViewport(_graph);

        //        debug("DIM: " + dim);



        double[] placeX = new double[_gw];

        for(int x = 0; x < _gw; x++) {

            placeX[x] = x*dim.getWidth()/_gw;

        }

        double[] placeY = new double[_gh];

        for(int y = 0; y < _gh; y++) {

            placeY[y] = y*dim.getWidth()/_gh;

        }



        Rectangle2D viewport = _target.getViewport(_graph);

        for(int i = 0; i < _gw; i++) {

            for(int j = 0; j < _gh; j++) {

                ASSERT(_minGrid != null, "Null min grid!");

                Node n = _minGrid[i][j];

                if(n != null) {

                    double x, y;

                    x = (_gw > 1) ? i*dim.getWidth()/(_gw-1)+dim.getX() :

                        dim.getX()+dim.getWidth()/2;

                    y = (_gh > 1) ? j*dim.getHeight()/(_gh-1)+dim.getY() :

                        dim.getY()+dim.getHeight()/2;

                    LayoutUtilities.placeNoReroute(_target, n, x, y);

                    //                    debug("Assigning " + n + ": " + x + ", " + y);

                }

            }

        }



        LayoutUtilities.routeVisibleEdges(_graph, _target);

    }

    



    

    /**

     * Cleanup the data structures used in the layout so that they are

     * empty next time the layout is called.

     */

    private void cleanupStructures() {

        _map = null;

        _target = null;

        _grid = null;

        _minGrid = null;

        _minCost = Double.MAX_VALUE;

        _gw = -1;

        _gh = -1;

    }

    

    /**

     * Debugging output.

     */

    private void debug(String s) {

       System.err.println(s);

    }



    /**

     * Return the absolute cost of an individual edge.  By default the

     * cost function is:

     *

     * <pre>

     *  EDGE_COST(e) = [

     *        HEIGHT(e) +

     *        WIDTH(e) +

     *        ELBOW_PENALTY(e) +

     *        EDGE_OVERLAP_PENALTY * num_overlap(e) +

     *        CROSSING_PENALTY * num_crossing(e)

     *  ]

     * </pre>

     */

    protected double edgeCost(Edge e) {

        Node tail = e.getTail();

        Node head = e.getHead();

        if(head == null || tail == null) {

            return 0;

        }

        

        int[] ptail = getXY(tail);

        int[] phead = getXY(head);

        double heightCost = HEIGHT_FACTOR * Math.abs(phead[1]-ptail[1]);

        double widthCost = WIDTH_FACTOR * Math.abs(phead[0]-ptail[0]);

        double elbowCost = (heightCost==0||widthCost==0) ? 0 : ELBOW_PENALTY;

        double overlapCost = numOverlaps(e, _graph) * EDGE_OVERLAP_PENALTY;

        double crossingCost = numCrossings(e, _graph) * CROSSING_PENALTY;

        return heightCost + widthCost + elbowCost + overlapCost + crossingCost;

    }

    

    /**

     * @see setCoolingFactor(double)

     */

    public double getCoolingFactor() {

        return _cool;

    }



    /**

     * @see setIterationCount(int)

     */

    public int getIterationCount(int cnt) {

        return _numIters;

    }



    /**

     * @see setMoveCount(int)

     */

    public int getMoveCount(int cnt) {

        return _numIters;

    }



    /**

     * Return the sparseness value of this layout; the default value

     * is 1.0.

     *

     * @see setSparseness(double)

     */

    public double getSparseness() {

        return _sparseness;

    }

    

    /**

     * Return the logical X, Y positions of the given

     * node as an integer array of length 2.

     */

    protected int[] getXY(Node n) {

        return (int[])_map.get(n);

    }



    /**

     * Initialize the grid and randomly assign nodes to vertices of

     * the grid.  The grid initialization is based on the aspect ratio

     * of the viewport in which the layout is being performed.  In

     * particular the following algorithm is used:

     *

     * <pre>

     *      GH = H/W * sqrt(N) * SPARSENESS

     * </pre>

     *

     * Where H and W are the height and width of the viewport, N is

     * the number of nodes in the graph, and SPARSENESS is some

     * measure of the sparseness of the layout.  A SPARSENESS of 1

     * will mean that the graph is tightly packed, and the packing

     * amount decreases linearly with the SPARSENESS value.

     *

     * @see setSparseness(double)

     */

    protected void initGrid() {

        if(_graph.getNodeCount() > 0) {

            Rectangle2D dim = _target.getViewport(_graph);

            double aspect = dim.getHeight()/dim.getWidth();

            double gh = Math.sqrt(_graph.getNodeCount()*aspect)*_sparseness;

            _gh = (int)Math.ceil(gh);

            _gw = (int)Math.ceil(gh/aspect);

            _grid = new Node[_gw][_gh];



            Iterator nodes = _graph.nodes();

            for(int x = 0; x < _gw; x++) {

                for(int y = 0; y < _gh; y++) {

                    if(!nodes.hasNext()) { break; }

                    Node n = (Node)nodes.next();

                    setXY(n, x, y);

                }

            }

        }

    }



    /**

     * Perform the annealing layout algorithm on the given graph

     * in the context of the given layout target.

     */

    public void layout(LayoutTarget target, Graph g) {

        _target = target;

        _graph = g;

        _map = new HashMap();



        if(_graph.getNodeCount() > 0) {

            initGrid();

            anneal();

            assignLayout();

            cleanupStructures();

        }

    }



    /**

     * Return the absolute cost of an individual node.  By default the

     * cost function is:

     *

     * <pre>

     *  NODE_COST(n) = SUM [ EDGE_COST(n.edge(i)) ] +

     *                 TEE_PENALTY * num_tee(g)

     * </pre>

     */

    protected double nodeCost(Node n) {

        if(n == null) {

            return 0;

        }



        int cost = 0;

        for(Iterator i = n.inEdges(); i.hasNext(); ) {

            cost += edgeCost((Edge)i.next());

        }

        for(Iterator i = n.outEdges(); i.hasNext(); ) {

            cost += edgeCost((Edge)i.next());

        }

        double teeCost = numTees(_graph) * TEE_PENALTY;

        cost += teeCost;

        

        return cost;

    }



    /**

     * Return the number of crossings between this edge and other

     * edges in the graph.

     */

    protected final int numCrossings(Edge in, Graph g) {

        int num = 0;

        Node inTail = in.getTail();

        Node inHead = in.getHead();



        if(inHead == null || inTail == null) {

            return 0;

        }

        

        int[] inTailPt = getXY(inTail);

        int[] inHeadPt = getXY(inHead);

        

        for(Iterator i = g.nodes(); i.hasNext(); ) {

            Node n = (Node)i.next();

            for(Iterator j = n.outEdges(); j.hasNext(); ) {

                Edge e = (Edge)j.next();

                Node t = e.getTail();

                Node h = e.getHead();

                if(t == null || h == null ||

                        t == inTail || t == inHead ||

                        h == inTail || h == inHead) {

                    //these cannot cross

                    continue;

                }

                int[] tailPt = getXY(t);

                int[] headPt = getXY(h);

                Line2D l =

                    new Line2D.Double(tailPt[0], tailPt[1],

                            headPt[0], headPt[1]);

                

                if(Line2D.linesIntersect(inTailPt[0], inTailPt[1],

                        inHeadPt[0], inHeadPt[1],

                        tailPt[0], tailPt[1],

                        headPt[0], headPt[1])) {

                    num++;

                }

            }

        }

        //        debug("\tEdge crossings: " + in + ", " + num);

        return num;

    }



    /**

     * Return the number of instances of nodes that are being placed

     * on top of edges that they are not connected to.

     */

    protected final int numTees(Graph g) {

        return 0; //XXX

    }

    

    /**

     * Return the number of overlaps between this edge and other edges

     * in the graph.  For now, simply test to see if the lines are

     * horizontal or vertical and overlap with each other.

     */

    protected final int numOverlaps(Edge in, Graph g) {

        int num = 0;

        Node inTail = in.getTail();

        Node inHead = in.getHead();



        if(inHead == null || inTail == null) {

            return 0;

        }

        

        int[] inTailPt = getXY(inTail);

        int[] inHeadPt = getXY(inHead);



        for(int which = 0; which < 2; which++) {

            if(inTailPt[which] == inHeadPt[which]) {

                for(Iterator i = g.nodes(); i.hasNext(); ) {

                    Node n = (Node)i.next();

                    for(Iterator j = n.outEdges(); j.hasNext(); ) {

                        Edge e = (Edge)j.next();

                        Node t = e.getTail();

                        Node h = e.getHead();

                        if(h != null && t != null) {

                            int[] tailPt = getXY(t);

                            int[] headPt = getXY(h);

                            if(tailPt[which] == headPt[which] &&

                                    tailPt[which] == inTailPt[which]) {

                                int other = which+1 % 2;

                                // test to see if the "other" coordinate is

                                // *between* the two points.

                                num++;

                            }

                        }

                    }

                }

                break;

            }

        }

        //        debug("\tEdge overlaps: " + in + ", " + num);

        return num;

    }



    /**

     * Set the cooling factor to be a value greater than 0 and less

     * than or equal to 1.  The cooling factor determines how quickly

     * the annealing "settles"; the lower the factor, the faster the

     * annealing settles.  The Default value is .95.  

     */

    public void setCoolingFactor(double val) {

        if(val <= 0 || val > 1) {

            String err = "Cooling factor must be greater than 0 and less or equal to 1: " + val;

            throw new IllegalArgumentException(err);

        }

        _cool = val;

    }

    

    /**

     * Set the number of iterations to cool over.  Default value is 100.

     */

    public void setIterationCount(int cnt) {

        _numIters = cnt; 

    }



    /**

     * Set the number of moves per iteration.  Default value is 10.

     */

    public void setMoveCount(int cnt) {

        _numIters = cnt; 

    }



    /**

     * Set the sparseness of this layout.  A sparseness of 1.0 will

     * mean that the graph is tightly packed, and the packing amount

     * decreases linearly with the SPARSENESS value.

     */

    public void setSparseness(double val) {

        if(val < 1.0) {

            String err = "Illegal sparseness value: " + val;

            throw new IllegalArgumentException(err);

        }

        _sparseness = val;

    }





    /**

     * Set the logical X, Y positions of the given node.

     */

    protected void setXY(Node n, int x, int y) {

        int[] pos = (int[])_map.get(n);

        if(pos == null) {

            pos = new int[2];

            _map.put(n, pos);

        }

        _grid[x][y] = n;

        pos[0] = x;

        pos[1] = y;

    }





    /**

     * Take a snapshot of the minimum cost arrangement of the nodes,

     * so that we can backtrack to it later.

     */

    private void snapMin() {

        if(_minGrid == null) {

            _minGrid = new Node[_gw][_gh];

        }

        for(int x = 0; x < _gw; x++) {

            for(int y = 0; y < _gh; y++) {

                _minGrid[x][y] = _grid[x][y];

            }

        }

    }



    /**

     * Swap the two nodes' grid positions.

     */

    private void swap(Node n1, Node n2) {

        //        debug("SWAP: " + n1 + ", " + n2);



        int xtmp, ytmp;

        int[] xy1 = getXY(n1);

        int[] xy2 = getXY(n2);

        xtmp = xy1[0];

        ytmp = xy1[1];

        setXY(n1, xy2[0], xy2[1]);

        setXY(n2, xtmp, ytmp);

    }

}



