/*
 * $Id: GraphEventMulticaster.java,v 1.1.1.1 2000/05/15 22:11:07 dauplait Exp $
 *
 * Copyright (c) 1998 The Regents of the University of California.
 * All rights reserved.  See the file COPYRIGHT for details.
 */
package diva.graph.model;
import java.util.*;

/**
 * A list of GraphListeners which is smart enough
 * to call the correct methods on these listeners
 * given a GraphEvent's ID.
 *
 * @author Michael Shilman  (michaels@eecs.berkeley.edu)
 * @version $Revision: 1.1.1.1 $
 * @rating Yellow
 */
public class GraphEventMulticaster implements GraphListener {
    /**
     * The list of graph listeners.
     */
    private List _listeners;

    /**
     * Create an empty multicaster object.
     */
    public GraphEventMulticaster() {
        _listeners = new LinkedList();
    }

    /**
     * Add the given listener to the list of listeners.
     */
    public void add(GraphListener l) {
        _listeners.add(l);
    }

    /**
     * Return an iterator over the list of listeners.
     */
    public Iterator listeners() {
        return _listeners.iterator();
    }

    /**
     * Dispatch an event to the list of listeners, calling
     * the appropriate method based on the event's ID.
     */
    public void dispatchEvent(GraphEvent e) {
        switch(e.getID()) {
            case GraphEvent.EDGE_ADDED:
                edgeAdded(e);
                break;
            case GraphEvent.EDGE_HEAD_CHANGED:
                edgeHeadChanged(e);
                break;
            case GraphEvent.EDGE_TAIL_CHANGED:
                edgeTailChanged(e);
                break;
            case GraphEvent.NODE_ADDED:
                nodeAdded(e);
                break;
            case GraphEvent.NODE_REMOVED:
                nodeRemoved(e);
                break;
            case GraphEvent.STRUCTURE_CHANGED:
                structureChanged(e);
                break;
        }
    }

    /**
     * Remove the given listener from the list
     * of listeners.
     */
    public void remove(GraphListener l) {
        _listeners.remove(l);
    }


    //==========================================================
    // GraphListener Implementation Follows
    //==========================================================


    public void nodeAdded(GraphEvent e) {
        for(Iterator i = listeners(); i.hasNext(); ) {
            GraphListener l = (GraphListener)i.next();
            l.nodeAdded(e);
        }
    }

    public void nodeRemoved(GraphEvent e) {
        for(Iterator i = listeners(); i.hasNext(); ) {
            GraphListener l = (GraphListener)i.next();
            l.nodeRemoved(e);
        }
    }

    public void structureChanged(GraphEvent e) {
        for(Iterator i = listeners(); i.hasNext(); ) {
            GraphListener l = (GraphListener)i.next();
            l.structureChanged(e);
        }
    }

    public void edgeAdded(GraphEvent e) {
        for(Iterator i = listeners(); i.hasNext(); ) {
            GraphListener l = (GraphListener)i.next();
            l.edgeAdded(e);
        }
    }
    
    public void edgeHeadChanged(GraphEvent e) {
        for(Iterator i = listeners(); i.hasNext(); ) {
            GraphListener l = (GraphListener)i.next();
            l.edgeHeadChanged(e);
        }
    }

    public void edgeTailChanged(GraphEvent e) {
        for(Iterator i = listeners(); i.hasNext(); ) {
            GraphListener l = (GraphListener)i.next();
            l.edgeTailChanged(e);
        }
    }
}
