/*
 * $Id: GraphImpl.java,v 1.1.1.1 2000/05/15 22:11:07 dauplait Exp $
 *
 * Copyright (c) 1998 The Regents of the University of California.
 * All rights reserved.  See the file COPYRIGHT for details.
 */
package diva.graph.model;

/**
 * A GraphImpl object provides creation and topology modification
 * methods for a particular graph implementation.
 *
 * The factory methods create graph elements (nodes, edges, and
 * graphs) that are compatible with one another and can be edited
 * using the add/removeNode() and setEdgeHead/Tail() methods.  All of
 * the create methods take in the semantic object so that
 * implementations can possibly use this to help determine the
 * specific type of node/graph/edge to create.  A standard
 * implementation will simply perform, e.g.
 *
 * <pre>
 *   public Node createNode(Object semanticObject) {
 *      Node n = new BasicNode();
 *      n.setSemanticObject(semanticObject);
 *      return n;
 *   }
 * </pre>
 *
 * @author Michael Shilman  (michaels@eecs.berkeley.edu)
 * @version $Revision: 1.1.1.1 $
 * @rating Yellow
 */
public interface GraphImpl {
    /**
     * Add a node to the given graph.
     */
    public void addNode(Node n, Graph parent);
    
    /**
     * Add a edge to the given graph.
     */
    public void addEdge(Edge e, Graph parent);
    /**
     * Return a new instance of a composite node with the given
     * semantic object.
     */
    public CompositeNode createCompositeNode(Object semanticObject);

    /**
     * Return a new instance of a node with the given semantic object.
     */
    public Node createNode(Object semanticObject);

    /**
     * Return a new instance of a graph with the given semantic
     * object.
     */
    public Graph createGraph(Object semanticObject);

    /**
     * Return a new instance of an edge with the given semantic
     * object.
     */
    public Edge createEdge(Object semanticObject);

    /**
     * Remove a node from the given graph.
     */
    public void removeNode(Node n);

    /**
     * Set the edge's head to the given node.
     */
    public void setEdgeHead(Edge e, Node head);
    
    /**
     * Set the edge's tail to the given node.
     */
    public void setEdgeTail(Edge e, Node tail);
}


