/*
 * $Id: GraphModel.java,v 1.1.1.1 2000/05/15 22:11:07 dauplait Exp $
 *
 * Copyright (c) 1998 The Regents of the University of California.
 * All rights reserved.  See the file COPYRIGHT for details.
 */
   package diva.graph.model;
   import java.io.*;
   import java.util.*;
   import diva.util.*;
   import java.beans.*;
   import javax.swing.event.EventListenerList;

/**
 * A graph model and registration point for GraphListeners. Actions
 * which are taken on a graph through a GraphModel object cause events
 * to be broadcast to the listeners; actions which are taken on the
 * graph directly (through the Node/Graph/Edge method calls or through
 * a GraphImpl) are "quiet".  Algorithms that wish to batch notify
 * listeners after a set of manipulations can also manually send graph
 * events via the model, using the dispatchGraphEvent() method. Graph
 * models also support user-object modification and property changes
 * and an associated set of listeners. <p>
 *
 * If for some reason the client tries to modify some graph elements
 * which do not belong to this model, the operations will still be
 * performed and the events will still be dispatched.  In this case,
 * it is the responsibility of the listeners to selectively ignore the
 * events.  The events that should be ingored are documented in the
 * GraphListener interface. <p>
 *
 * As a policy there is a single graph model which sits above the root
 * of a (possibly hierarchically nested) graph.  However in the future
 * we will allow GraphModels to be created on sub-graphs of the root
 * graph to give the user a "window" onto some subset of the
 * hierarchy.  When a listener registers on one of these sub-models,
 * he will only receive events relevant to that submodel.  What is
 * "relevant" can be expressed somewhat formally: <p>
 *
 * <PRE>
 * 1)  [event in model1] & [model1's graph is subgraph of model2's graph]
 *         ==> [event in model2]
 * 2)  [event in model2 in model1's graph or a subgraph thereof]
 *         ==> [event in model2]
 * </PRE>
 *
 * For example, consider a graph model TOP which wraps a graph
 * A, and a graph model SUB which wraps B, where B is a subgraph
 * of A: <p>
 *
 * <PRE>
 *    TOP
 *       \_A
 *         |
 *         ...
 *   SUB   |
 *      \__B
 *         |
 *         ...
 * </PRE>
 *
 * In this case, the semantics dictate that all events generated by
 * SUB should also be heard by TOP's listeners.  Also, all events from
 * TOP which occur on objects below B should be heard by SUB's
 * listeners.
 *
 * @see GraphListener
 * @see GraphImpl
 * @author Michael Shilman (michaels@eecs.berkeley.edu)
 * @author John Reekie     (johnr@eecs.berkeley.edu)
 * @version $Revision: 1.1.1.1 $
 * @rating Yellow
 */
   public class GraphModel {
    /**
     * The factory that is used to construct new graph elements that
     * are compatible with this model.
     */
      private GraphImpl _graphImpl = null;
   
    /**
     * The graph that this is a model of.
     */
      private Graph _graph = null;
   
    /**
     * Whether or not to dispatch events.
     */
      private boolean _dispatch = true;
   
    /**
     * The list of graph listeners.
     */
      private GraphEventMulticaster _graphListeners;
   
    /**
     * The list of property and user object listeners.
     */
      private EventListenerList _otherListeners;
   
    /**
     * Construct a new top-level graph model with an empty graph of
     * type BasicGraph.
     */
      public GraphModel() {
         this(new BasicGraphImpl());
      }
   
    /**
     * Construct a new top-level graph model with the given graph
     * editor and factory, which are used to modify the topology of
     * the graph and create new graph elements respectively.  The
     * given editor and factory must be "compatible" (i.e. the
     * elements returned by the factory must be of the type assumed by
     * the given editor) and this is up to the user to ensure.
     */
      public GraphModel(GraphImpl gi) {
         _graphImpl = gi;
         _graphListeners = new GraphEventMulticaster();
         _otherListeners = new EventListenerList();
         _graph = _graphImpl.createGraph(null);
      }
   
   
    /**
     * Add a graph listener to the model.  Graph listeners are
     * notified with a GraphEvent any time a graph manipulation method
     * is performed using this model.
     */
      public void addGraphListener(GraphListener l) {
         _graphListeners.add(l);
      }
   
    /**
     * Add a node to the graph directly contained by this model; a
     * shortcut to addNode(n, getGraph()), which can be useful for
     * the common case of graphs that are note hierarchically nested.
     */
      public void addNode(Node n) {
         addNode(n, getGraph());
      }
   
    /**
     * Add a node to the given graph and notify listeners with a
     * NODE_ADDED event. <p>
     */
      public void addNode(Node n, Graph parent) {
         Graph prevParent = n.getParent();
         GraphEvent e = new GraphEvent(GraphEvent.NODE_ADDED,
                                       this, n, prevParent);
         _graphImpl.addNode(n, parent);
         dispatchGraphEvent(e);
      } 
        
    /**
     * Add a edge to the graph directly contained by this model; a
     * shortcut to addEdge(e, getGraph()), which can be useful for
     * the common case of graphs that are note hierarchically nested.
     * @author Claudia
     */
      public void addEdge(Edge e) {
         addEdge(e, getGraph());
      }
   
    /**
     * Add a edge to the given graph and notify listeners with a
     * EDGE_ADDED event. <p>
     * @author Claudia
     */
      public void addEdge(Edge ed, Graph parent) {
         GraphEvent e = new GraphEvent(GraphEvent.EDGE_ADDED,
                                       this, ed, parent);
         _graphImpl.addEdge(ed, parent);
         dispatchGraphEvent(e);
      }
   
    /**
     * Add a property listener to the model.  Property listeners are
     * notified with an PropertyChangeEvent any time setProperty() is
     * called.
     */
      public void addPropertyChangeListener(PropertyChangeListener l) {
         _otherListeners.add(PropertyChangeListener.class, l);
      }
   
    /**
     * Add a user object listener to the model.  User object
     * listeners are notified with an SemanticObjectChangeEvent any time
     * setSemanticObject() is called.
     */
      public void addSemanticObjectChangeListener(SemanticObjectChangeListener l) {
         _otherListeners.add(SemanticObjectChangeListener.class, l);
      }
   
    /**
     * Connect the given edge to the given tail and head nodes,
     * then dispatch events to the listeners.
     */
      public void connectEdge(Edge e, Node tail, Node head) {
         Node prevHead = e.getHead();
         Node prevTail = e.getTail();
         _graphImpl.setEdgeTail(e, tail);
         _graphImpl.setEdgeHead(e, head);
         addEdge(e);
      
        //set everything up first, THEN dispatch events
         dispatchGraphEvent(new GraphEvent(GraphEvent.EDGE_HEAD_CHANGED,
                                           this, e, prevHead));
         dispatchGraphEvent(new GraphEvent(GraphEvent.EDGE_TAIL_CHANGED,
                                           this, e, prevTail));
      }
   
   
    /**
     * Return a new instance of a node that is compatible with this
     * particular model, but do not add it to the graph.  It is also
     * possible for clients to create nodes manually, or for
     * particular model implementations to add other factory
     * constructors. <p>
     *
     * This method is useful for clients who know little about the
     * specific model, but want to properly add nodes to the model.
     * For example, certain graph layout algorithms use this method to
     * create dummy nodes during layout.
     *
     * @deprecated 
     */
      public CompositeNode createCompositeNode() {
         return createCompositeNode(null);
      }
   
    /**
     * Return a new instance of a node that is compatible with this
     * particular model, but do not add it to the graph.  It is also
     * possible for clients to create nodes manually, or for
     * particular model implementations to add other factory
     * constructors. <p>
     */
      public CompositeNode createCompositeNode(Object semanticObject) {
         return _graphImpl.createCompositeNode(semanticObject);
      }
   
    /**
     * Return a new instance of an edge that is compatible with this
     * particular model.
     *
     * @deprecated 
     */
      public Edge createEdge() {
         return createEdge(null);
      }
   
    /**
     * Return a new instance of an edge that is compatible with this
     * particular model, connecting given head and tail nodes, and
     * with the given user object.
     * Shorthand for "e = createEdge(); e.setSemanticObject(semanticObject);
     * connectEdge(e, tail, head);".
     *
     * @deprecated 
     */
      public Edge createEdge(Object semanticObject, Node tail, Node head) {
         Edge e = createEdge(semanticObject);
         connectEdge(e, tail, head);
         return e;
      }
   
    /**
     * Return a new instance of an edge that is compatible with this
     * particular model, with the given user object.
     */
      public Edge createEdge(Object semanticObject) {
         return _graphImpl.createEdge(semanticObject);
      }
   
    /**
     * Return a new instance of an edge that is compatible with this
     * particular model, connecting given head and tail nodes.
     * Shorthand for "e = createEdge(); connectEdge(e, tail, head);".
     *
     * @deprecated
     */
      public Edge createEdge(Node tail, Node head) {
         return createEdge(null, tail, head);
      }
   
    /**
     * Return a new instance of a node that is compatible
     * with this particular model, but do not add it to the
     * graph.  
     *
     * This method is useful for clients who know little about
     * the specific model, but want to properly add nodes to the model.
     * For example, certain graph layout algorithms use
     * this method to create dummy nodes during layout.
     *
     * @deprecated
     */
      public Node createNode() {
         return createNode(null);
      }
   
    /**
     * Return a new instance of a node that is compatible with this
     * particular model with the given user object, but do not add it
     * to the graph.  It is also possible for clients to create nodes
     * manually, or for particular model implementations to add other
     * factory constructors. <p>
     */
      public Node createNode(Object semanticObject) {
         return _graphImpl.createNode(semanticObject);
      }
   
    /**
     * Send a semantic object change event to all of the appropriate
     * listeners.
     */
      public void dispatchSemanticObjectChangeEvent(SemanticObjectChangeEvent e) {
         if(_dispatch) {
            Object[] listeners = _otherListeners.getListenerList();
            for (int i = listeners.length-2; i>=0; i-=2) {
               if (listeners[i] == SemanticObjectChangeListener.class) {
                  ((SemanticObjectChangeListener)listeners[i+1]).objectChange(e);
               }
            }
         }
      }
   
    /**
     * Send a property event to all of the property listeners.
     */
      public void dispatchPropertyChangeEvent(PropertyChangeEvent e) {
         if(_dispatch) {
            Object[] listeners = _otherListeners.getListenerList();
            for (int i = listeners.length-2; i>=0; i-=2) {
               if (listeners[i] == PropertyChangeListener.class) {
                  ((PropertyChangeListener)listeners[i+1]).propertyChange(e);
               }
            }
         }
      }
   
    /**
     * Send an graph event to all of the graph listeners.
     */
      public void dispatchGraphEvent(GraphEvent e) {
         if(_dispatch) {
            _graphListeners.dispatchEvent(e);
         }
      }
   
    /**
     * Disconnect an edge from its two enpoints and notify graph
     * listeners with an EDGE_HEAD_CHANGED and an EDGE_TAIL_CHANGED
     * event.
     */
      public void disconnectEdge(Edge e) {
         setEdgeTail(e, null);
         setEdgeHead(e, null);
      }
   
    /**
     * Return the graph that is contained by this model.  As a policy this
     * is set in the constructor of the GraphModel implementation
     * so that it cannot be externally modified in the lifetime
     * of the model.
     */
      public Graph getGraph() {
         return _graph;
      }
   
    /**
     * Set the graph.
     * @author Claudia
     */
      public void setGraph(Graph g) {
         _graph = g;
      }
   
    /**
     * Return the graph implementation of this model.
     */
      public GraphImpl getGraphImpl() {
         return _graphImpl;
      }
   
   
    /**
     * Remove a graph listener from the model.
     */
      public void removeGraphListener(GraphListener l) {
         _graphListeners.remove(l);
      }
   
    /**
     * Remove a property listener from the model.
     */
      public void removePropertyChangeListener(PropertyChangeListener l) {
         _otherListeners.remove(PropertyChangeListener.class, l);
      }
   
    /**
     * Remove a semantic object change listener from the model.
     */
      public void removeSemanticObjectChangeListener(SemanticObjectChangeListener l) {
         _otherListeners.remove(SemanticObjectChangeListener.class, l);
      }
   
    /**
     * Delete a node from the graph and notify graph listeners with a
     * NODE_REMOVED event.
     */
      public void removeNode(Node n) {
         Graph parent = n.getParent();
         _graphImpl.removeNode(n);
         dispatchGraphEvent(new GraphEvent(GraphEvent.NODE_REMOVED,
                                           this, n, parent));
      }
   
    /**
     * Turn on/off all event dispatches from this graph model, for use
     * in an inner-loop algorithm.  When turning dispatch back on
     * again, if the client has made changes that listeners should
     * know about, he should create an appropriate STRUCTURE_CHANGED
     * and dispatch it using the dispatchGraphEvent() method.
     */
      public void setDispatchEnabled(boolean val) {
         _dispatch = val;
      }
   
    /**
     * Connect an edge to the given head node and notify listeners
     * with an EDGE_HEAD_CHANGED event.
     */
      public void setEdgeHead(Edge e, Node head) {
         Node prevHead = e.getHead();
         _graphImpl.setEdgeHead(e, head);
         dispatchGraphEvent(new GraphEvent(GraphEvent.EDGE_HEAD_CHANGED,
                                           this, e, prevHead));
      }
   
    /**
     * Connect an edge to the given tail node and notify listeners
     * with an EDGE_TAIL_CHANGED event.
     */
      public void setEdgeTail(Edge e, Node tail) {
         Node prevTail = e.getTail();
         _graphImpl.setEdgeTail(e, tail);
         dispatchGraphEvent(new GraphEvent(GraphEvent.EDGE_TAIL_CHANGED,
                                           this, e, prevTail));
      }
   
    /**
     * Set a property on an edge and dispatch a PropertyChangedEvent
     * to property listeners.
     *
     * @param e             The edge whose property is being set.
     * @param propName      The name of the property that is being set.
     * @param o             The new value for the property.
     */
      public void setProperty(PropertyContainer pc, String propName, Object o) {
         Object prev = pc.getProperty(propName);
         pc.setProperty(propName, o);
         dispatchPropertyChangeEvent(new PropertyChangeEvent(pc, propName, prev, o));
      }
   
    /**
     * Set the user object of a user object container and notify
     * user object listeners with a SemanticObjectChangeEvent event.
     */
      public void setSemanticObject(SemanticObjectContainer uoc, Object o) {
         Object prev = uoc.getSemanticObject();
         uoc.setSemanticObject(o);
         SemanticObjectChangeEvent e = new SemanticObjectChangeEvent(uoc, prev);
         dispatchSemanticObjectChangeEvent(e);
      }
   }




