/*

 * $Id: GraphUtilities.java,v 1.1.1.1 2000/05/15 22:11:07 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */



package diva.graph.model;

import java.util.AbstractSequentialList;

import java.util.Iterator;

import diva.util.IteratorAdapter;

import diva.util.FilteredIterator;

import diva.util.Filter;



/**

 * A set of utilities for traversing/manipulating/etc. graphs.

 *

 * @author Michael Shilman (michaels@eecs.berkeley.edu)

 * @version $Revision: 1.1.1.1 $

 * @rating Red

 */

public final class GraphUtilities {



    //You can't instantiate this class

    private GraphUtilities() {}



    /**

     * Return an iterator over the nodes on the

     * other side of edges arriving in the given node.

     */

    public static final Iterator inNodes(Node n) {

        return new diva.util.ProxyIterator(n.inEdges()) {

	        public Object next() {

	            Edge e = (Edge)super.next();

	            return e.getTail();

	        }

	    };

    }



    /**

     * Return an iterator over the nodes on the

     * other side of edges emanating from the given node.

     */

    public static final Iterator outNodes(Node n) {

        return new diva.util.ProxyIterator(n.outEdges()) {

	        public Object next() {

	            Edge e = (Edge)super.next();

	            return e.getHead();

	        }

	    };

    }



    /**

     * Disconnect all of the edges connected to the given node, then

     * remove it from its graph.

     */

    public static final void purgeNode(GraphImpl impl, Node n) {

        for(Iterator i = n.outEdges(); i.hasNext(); ) {

            Edge e = (Edge)i.next();

            impl.setEdgeTail(e, null);

            impl.setEdgeHead(e, null);

        }

        for(Iterator i = n.inEdges(); i.hasNext(); ) {

            Edge e = (Edge)i.next();

            impl.setEdgeTail(e, null);

            impl.setEdgeHead(e, null);

        }

        impl.removeNode(n);

    }



    /**

     * Disconnect all of the edges connected to the given node, then

     * remove it from its graph.  Generate events so that clients

     * can update their views.

     */

    public static final void purgeNode(GraphModel m, Node n) {

        for(Iterator i = n.outEdges(); i.hasNext(); ) {

            Edge e = (Edge)i.next();

            m.disconnectEdge(e);

        }

        for(Iterator i = n.inEdges(); i.hasNext(); ) {

            Edge e = (Edge)i.next();

            m.disconnectEdge(e);

        }

        m.removeNode(n);

    }

    

    /**

     * Mark all the nodes in a graph as visited

     * according to the given value.

     */

    public static final void setAllVisited(Graph g, boolean visited) {

        for(Iterator i = g.nodes(); i.hasNext(); ) {

            Node n = (Node)i.next();

            n.setVisited(visited);

        }

    }



    /**

     * Check to make sure that all nodes and edges

     * are self-consistent within a graph.

     */

    public static final boolean checkConsistency(Graph g) {

        for(Iterator i = g.nodes(); i.hasNext(); ) {

            Node n = (Node)i.next();

            for(Iterator j = n.outEdges(); j.hasNext(); ) {

                Edge e = (Edge)j.next();

                if((e == null) || (e.getHead() == null)) {

                    return false;

                }

            }

            for(Iterator j = n.inEdges(); j.hasNext(); ) {

                Edge e = (Edge)j.next();

                if((e == null) || (e.getTail() == null) ){

                    return false;

                }

            }

        }

        return true;

    }



    /**

     * Check to make sure that all nodes and edges

     * are contained with in a graph and are consistent.

     */

    public static final boolean checkContainment(Graph g) {

        if(!checkConsistency(g)) {

            return false;

        }



        for(Iterator i = g.nodes(); i.hasNext(); ) {

            Node n = (Node)i.next();

            if(n.getParent() != g) {

                return false;

            }

            for(Iterator j = n.outEdges(); j.hasNext(); ) {

                Edge e = (Edge)j.next();

                if(e.getHead().getParent() != g) {

                    return false;

                }

            }

            for(Iterator j = n.inEdges(); j.hasNext(); ) {

                Edge e = (Edge)j.next();

                if(e.getTail().getParent() != g) {

                    return false;

                }

            }

        }

        return true;

    }



    /**

     * Return an iterator over the edges in a graph

     * which are local to that graph (i.e. the edges

     * whose head and tail nodes are both in the

     * graph.

     */

    public static final Iterator localEdges(Graph g) {

        return new EdgeIterator(g, new LocalEdgeFilter(g));

    }



    public static final class EdgeIterator extends IteratorAdapter {

        private Iterator _nodeIter;

        private Iterator _edgeIter;

        private Filter _filter;

        private Object _nextEdge;



        public EdgeIterator(Graph g) {

            _nodeIter = g.nodes();

        }



        public EdgeIterator(Graph g, Filter f) {

            _nodeIter = g.nodes();

            _filter = f;

        }



        public boolean hasNext() {

            if(_nextEdge != null) {

                return true;

            }



            getNext();

            if(_nextEdge != null) {

                return true;

            }

            return false;

        }



        public Object next() {

            if(_nextEdge == null) {

                getNext();

            }

            Object result = _nextEdge;

            _nextEdge = null;

            return result;

        }



        private void getNext() {

            while(_nextEdge == null) {

                if(_edgeIter == null) {

                    if(_nodeIter.hasNext()) {

                        Iterator outEdges = ((Node)_nodeIter.next()).outEdges();

                        if(_filter == null) {

                            _edgeIter = outEdges;

                        }

                        else {

                            _edgeIter = new FilteredIterator(outEdges, _filter);

                        }

                    }

                    else {

                        break;

                    }

                }



                if(_edgeIter.hasNext()) {

                    _nextEdge = _edgeIter.next();

                }

                else {

                    _edgeIter = null;

                }

            }

        }

    }



    private static final class LocalEdgeFilter implements Filter {

        private Graph _graph;



        public LocalEdgeFilter(Graph g) {

            _graph = g;

        }



        public boolean accept(Object o) {

            if(o instanceof Edge) {

                Edge e = (Edge)o;

                return ((e.getHead() != null) && (e.getHead().getParent() == _graph) &&

                    (e.getTail() != null) && (e.getTail().getParent() == _graph));

            }

            else {

                return false;

            }

        }

    }



    /*

     * Perform a topological sort over the nodes in the

     * given directed, acyclic graph, and place the results

     * in the given "results" list.  This method throws an

     * illegal argument exception if the graph is not directed

     * or has cycles.

     *

    public void topologicalSort(Graph g, AbstractSequentialList results)

        throws IllegalArgumentException {



    }*/

}

