/*

 * $Id: PaintedShape.java,v 1.1.1.1 2000/05/15 22:11:08 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 *

 */



package diva.util.java2d;



import java.awt.BasicStroke;

import java.awt.Color;

import java.awt.Graphics2D;

import java.awt.Paint;

import java.awt.Shape;

import java.awt.Stroke;

import java.awt.geom.Rectangle2D;



/** A utility class that paints a shape with an stroke and/or fill.

 * This class is intended for use as a low-level class to simplify

 * construction of drawn graphics.  It contains a number of fields that

 * govern how the shape is painted, such as the fill paint, stroke

 * stroke or line width, and stroke paint.

 *

 * @version	$Revision: 1.1.1.1 $

 * @author 	John Reekie

 */

public class PaintedShape implements PaintedObject {



    /** The stroke

     */

    public BasicStroke stroke;



    /** The paint for the fill.

     */

    public Paint fillPaint;



    /** The shape being painted.

     */

    public Shape shape;



    /** The stroke paint

     */

    public Paint strokePaint;





    /** Create a painted shape on the given Shape. The shape

     * will be painted as an outline only.

     */

    public PaintedShape (Shape shape) {

        this.shape = shape;

        this.strokePaint = Color.black;

        setLineWidth(1.0f);

    }



    /** Create a painted shape on the given Shape. The shape

     * will be filled with the given paint, but not painted with

     * an outline.

     */

    public PaintedShape (Shape shape, Paint fillPaint) {

        this.shape = shape;

        this.fillPaint = fillPaint;

    }



    /** Create a painted shape on the given Shape. The shape

     * will be filled with the given paint and outlined with

     * a black outline of the given width.

     */

    public PaintedShape (Shape shape, Paint fillPaint, float lineWidth) {

        this.shape = shape;

        this.fillPaint = fillPaint;

        this.strokePaint = Color.black;

        setLineWidth(lineWidth);

    }



    /** Get the bounding box of the shape when painted. This method takes

     * account of the thickness of the stroke, if there is one.

     */

    public Rectangle2D getBounds () {

        if (stroke == null) {

            return shape.getBounds2D();

        } else {

            return stroke.createStrokedShape(shape).getBounds2D();

        }

    }



    /** Get the line width

     */

    public float getLineWidth () {

        if (stroke == null) {

            return 0.0f;

        } else {

            return stroke.getLineWidth();

        }

    }



    /** Get the stroke.

     */

    public BasicStroke getStroke () {

        return stroke;

    }



    /** Test if this shape intersects the given rectangle. Currently

     * this does not take into account the width of the stroke

     * or other things such as dashes, because of problems with

     * geometry testing with GeneralPath in the first version of

     * JDK1.2.

     */

    public boolean hit (Rectangle2D r) {

        // Hit testing on strokes doesn't appear to work too

        // well in JDK1.2, so we will cheat and ignore the width

        // of the stroke

        if (fillPaint != null) {

            // There's a fill, so test the internal of the path

            return shape.intersects(r);

        } else {

            // Otherwise test for intersection with the path

            return shape.intersects(r);

        }

    }



    /** Paint the shape. The shape is redrawn with the current

     *  shape, fill, and stroke.

     */

    public void paint (Graphics2D g) {

        // Fill it

        if (fillPaint != null) {

            g.setPaint(fillPaint);

            g.fill(shape);

        }

        // Stroke it

        if (stroke != null && strokePaint != null) {

            g.setStroke(stroke);

            g.setPaint(strokePaint);

            g.draw(shape);

        }

    }



    /** Set the line width. If the line width is less than

     * or equal to zero, the stroke will be removed.

     */

    public void setLineWidth (float lineWidth) {

        if (lineWidth <= 0.0) {

            stroke = null;

        } else {

            if (stroke == null || lineWidth != stroke.getLineWidth()) {

                stroke = PaintedPath.getStroke(lineWidth);

            }

        }

    }

}

